import random
import unittest
from robot.output.loggerhelper import LEVELS

from robot.reporting.jsmodelbuilders import JsBuildingContext
from robot.utils.asserts import assert_equal


class TestStringContext(unittest.TestCase):

    def test_add_empty_string(self):
        self._verify([''], [0], [])

    def test_add_string(self):
        self._verify(['Hello!'], [1], ['Hello!'])

    def test_add_several_strings(self):
        self._verify(['Hello!', 'Foo'], [1, 2], ['Hello!', 'Foo'])

    def test_cache_strings(self):
        self._verify(['Foo', '', 'Foo', 'Foo', ''], [1, 0, 1, 1, 0], ['Foo'])

    def test_escape_strings(self):
        self._verify(['</script>', '&', '&'], [1, 2, 2], ['&lt;/script&gt;', '&amp;'])

    def test_no_escape(self):
        self._verify(['</script>', '&', '&'], [1, 2, 2], ['</script>', '&'], escape=False)

    def test_none_string(self):
        self._verify([None, '', None], [0, 0, 0], [])

    def _verify(self, strings, exp_ids, exp_strings, escape=True):
        exp_strings = tuple('*'+s for s in [''] + exp_strings)
        ctx = JsBuildingContext()
        results = [ctx.string(s, escape=escape) for s in strings]
        assert_equal(results, exp_ids)
        assert_equal(ctx.strings, exp_strings)


class TestTimestamp(unittest.TestCase):

    def setUp(self):
        self._context = JsBuildingContext()

    def test_timestamp(self):
        assert_equal(self._context.timestamp('20110603 12:00:00.042'), 0)
        assert_equal(self._context.timestamp('20110603 12:00:00.043'), 1)
        assert_equal(self._context.timestamp('20110603 12:00:00.000'), -42)
        assert_equal(self._context.timestamp('20110603 12:00:01.041'), 999)
        assert_equal(self._context.timestamp('20110604 12:00:00.042'),
                      24 * 60 * 60 * 1000)

    def test_none_timestamp(self):
        assert_equal(self._context.timestamp(None), None)


class TestMinLogLevel(unittest.TestCase):

    def setUp(self):
        self._context = JsBuildingContext()

    def test_trace_is_identified_as_smallest_log_level(self):
        self._messages(list(LEVELS))
        assert_equal('TRACE', self._context.min_level)

    def test_debug_is_identified_when_no_trace(self):
        self._messages([l for l in LEVELS if l != 'TRACE'])
        assert_equal('DEBUG', self._context.min_level)

    def test_info_is_smallest_when_no_debug_or_trace(self):
        self._messages(['INFO', 'WARN', 'ERROR', 'FAIL'])
        assert_equal('INFO', self._context.min_level)

    def _messages(self, levels):
        levels = levels[:]
        random.shuffle(levels)
        for level in levels:
            self._context.message_level(level)


if __name__ == '__main__':
    unittest.main()



from robot.api.deco import keyword

ROBOT_AUTO_KEYWORDS = False


def public_method_is_not_keyword():
    raise RuntimeError('Should not be executed!')


@keyword(name="Decorated Method In Module Is Keyword")
def decorated_method():
    print('Decorated methods are keywords.')


def _private_method_is_not_keyword():
    raise RuntimeError('Should not be executed!')


@keyword
def _private_decorated_method_in_module_is_keyword():
    print('Decorated private methods are keywords.')



class library_with_keywords_with_dots_in_name:

    def get_keyword_names(self):
        return ['Dots.in.name.in.a.library',
                'Multiple...dots . . in . a............row.in.a.library',
                'Ending with a dot. In a library.']

    def run_keyword(self, name, args):
        return '-'.join(args)



import unittest
from os.path import dirname, join

from robot.api.parsing import get_model
from robot.result import ExecutionResult
from robot.model import SuiteVisitor, TestSuite
from robot.result import TestSuite as ResultSuite
from robot.running import TestSuite as RunningSuite
from robot.utils.asserts import assert_equal


RESULT = ExecutionResult(join(dirname(__file__), 'golden.xml'))


class TestVisitingSuite(unittest.TestCase):

    def setUp(self):
        self.suite = suite = TestSuite()
        suite.setup.config(name='SS')
        suite.teardown.config(name='ST')
        test = suite.tests.create()
        test.setup.config(name='TS')
        test.teardown.config(name='TT')
        test.body.create_keyword()

    def test_abstract_visitor(self):
        RESULT.suite.visit(SuiteVisitor())
        RESULT.suite.visit(SuiteVisitor())

    def test_start_suite_can_stop_visiting(self):
        RESULT.suite.visit(StartSuiteStopping())

    def test_start_test_can_stop_visiting(self):
        RESULT.suite.visit(StartTestStopping())

    def test_start_keyword_can_stop_visiting(self):
        RESULT.suite.visit(StartKeywordStopping())

    def test_visit_setups_and_teardowns(self):
        visitor = VisitSetupsAndTeardowns()
        self.suite.visit(visitor)
        assert_equal(visitor.visited, ['SS', 'TS', 'TT', 'ST'])

    def test_visit_keyword_teardown(self):
        suite = ResultSuite()
        suite.setup.config(kwname='SS')
        suite.teardown.config(kwname='ST')
        test = suite.tests.create()
        test.setup.config(kwname='TS')
        test.teardown.config(kwname='TT')
        test.body.create_keyword().teardown.config(kwname='KT')
        visitor = VisitSetupsAndTeardowns()
        suite.visit(visitor)
        assert_equal(visitor.visited, ['SS', 'TS', 'KT', 'TT', 'ST'])

    def test_dont_visit_inactive_setups_and_teardowns(self):
        suite = ResultSuite()
        suite.tests.create().body.create_keyword()
        visitor = VisitSetupsAndTeardowns()
        suite.visit(visitor)
        assert_equal(visitor.visited, [])

    def test_visit_for(self):
        class VisitFor(SuiteVisitor):
            in_for = False

            def start_for(self, for_):
                for_.variables = ['${y}']
                for_.flavor = 'IN RANGE'
                self.in_for = True

            def end_for(self, for_):
                for_.values = ['10']
                self.in_for = False

            def start_keyword(self, keyword):
                if self.in_for:
                    keyword.name = 'IN FOR'

        for_ = self.suite.tests[0].body.create_for(['${x}'], 'IN', ['a', 'b', 'c'])
        kw = for_.body.create_keyword(name='K')
        self.suite.visit(VisitFor())
        assert_equal(str(for_), 'FOR    ${y}    IN RANGE    10')
        assert_equal(kw.name, 'IN FOR')

    def test_visit_if(self):
        class VisitIf(SuiteVisitor):
            level = None

            def start_if(self, if_):
                self.level = 0

            def start_if_branch(self, branch):
                self.level += 1
                branch.body.create_keyword()

            def end_if_branch(self, branch):
                if branch.type != branch.ELSE:
                    branch.condition = 'x > %d' % self.level

            def end_if(self, if_):
                self.level = None

            def start_keyword(self, keyword):
                if self.level is not None:
                    keyword.name = 'kw %d' % self.level

        if_ = self.suite.tests[0].body.create_if()
        branch1 = if_.body.create_branch(if_.IF, condition='xxx')
        branch2 = if_.body.create_branch(if_.ELSE_IF, condition='yyy')
        branch3 = if_.body.create_branch(if_.ELSE)
        self.suite.visit(VisitIf())
        assert_equal(branch1.condition, 'x > 1')
        assert_equal(branch1.body[0].name, 'kw 1')
        assert_equal(branch2.condition, 'x > 2')
        assert_equal(branch2.body[0].name, 'kw 2')
        assert_equal(branch3.condition, None)
        assert_equal(branch3.body[0].name, 'kw 3')

    def test_start_and_end_methods_can_add_items(self):
        suite = RESULT.suite.deepcopy()
        suite.visit(ItemAdder())
        assert_equal(len(suite.tests), len(RESULT.suite.tests) + 2)
        assert_equal(suite.tests[-2].name, 'Added by start_test')
        assert_equal(suite.tests[-1].name, 'Added by end_test')
        assert_equal(len(suite.tests[0].body),
                     len(RESULT.suite.tests[0].body) + 2)
        assert_equal(suite.tests[0].body[-2].name, 'Added by start_keyword')
        assert_equal(suite.tests[0].body[-1].name, 'Added by end_keyword')

    def test_start_end_body_item(self):
        class Visitor(SuiteVisitor):
            def __init__(self):
                self.visited = []

            def start_body_item(self, item):
                self.visited.append(f'START {item.type}')

            def end_body_item(self, item):
                self.visited.append(f'END {item.type}')

        visitor = Visitor()
        RunningSuite.from_model(get_model('''
*** Test Cases ***
Example
    IF    True
        WHILE    True
            BREAK
        END
    ELSE IF    True
        FOR    ${x}    IN    @{stuff}
            CONTINUE
        END
    ELSE
        TRY
            Keyword
        EXCEPT    Something
            Keyword
        ELSE
            Keyword
        FINALLY
            Keyword
        END
    END
''')).visit(visitor)
        expected = '''
START IF/ELSE ROOT
    START IF
        START WHILE
            START BREAK
            END BREAK
        END WHILE
    END IF
    START ELSE IF
        START FOR
            START CONTINUE
            END CONTINUE
        END FOR
    END ELSE IF
    START ELSE
        START TRY/EXCEPT ROOT
            START TRY
                START KEYWORD
                END KEYWORD
            END TRY
            START EXCEPT
                START KEYWORD
                END KEYWORD
            END EXCEPT
            START ELSE
                START KEYWORD
                END KEYWORD
            END ELSE
            START FINALLY
                START KEYWORD
                END KEYWORD
            END FINALLY
        END TRY/EXCEPT ROOT
    END ELSE
END IF/ELSE ROOT
'''.strip().splitlines()
        assert_equal(visitor.visited, [e.strip() for e in expected])

    def test_visit_return_continue_and_break(self):
        suite = ResultSuite()
        suite.tests.create().body.create_return().body.create_keyword(kwname='R')
        suite.tests.create().body.create_continue().body.create_message(message='C')
        suite.tests.create().body.create_break().body.create_keyword(kwname='B')

        class Visitor(SuiteVisitor):
            visited_return = visited_continue = visited_break = False
            visited_return_body = visited_continue_body = visited_break_body = False

            def start_return(self, return_):
                self.visited_return = True

            def end_continue(self, continue_):
                self.visited_continue = True

            def start_break(self, break_):
                self.visited_break = True

            def start_keyword(self, keyword):
                if keyword.name == 'R':
                    self.visited_return_body = True
                if keyword.name == 'B':
                    self.visited_break_body = True

            def visit_message(self, msg):
                if msg.message == 'C':
                    self.visited_continue_body = True

        visitor = Visitor()
        suite.visit(visitor)
        for visited in 'return', 'continue', 'break':
            assert_equal(getattr(visitor, f'visited_{visited}'), True, visited)
            assert_equal(getattr(visitor, f'visited_{visited}_body'), True, f'{visited}_body')


class StartSuiteStopping(SuiteVisitor):

    def start_suite(self, suite):
        return False

    def end_suite(self, suite):
        raise AssertionError

    def start_test(self, test):
        raise AssertionError

    def start_keyword(self, keyword):
        raise AssertionError


class StartTestStopping(SuiteVisitor):

    def __init__(self):
        self.test_started = False

    def start_test(self, test):
        self.test_started = True
        return False

    def end_test(self, test):
        raise AssertionError

    def start_keyword(self, keyword):
        if self.test_started:
            raise AssertionError


class StartKeywordStopping(SuiteVisitor):

    def start_keyword(self, test):
        return False

    def end_keyword(self, test):
        raise AssertionError

    def log_message(self, msg):
        raise AssertionError


class VisitSetupsAndTeardowns(SuiteVisitor):

    def __init__(self):
        self.visited = []

    def start_keyword(self, keyword):
        if keyword.type in (keyword.SETUP, keyword.TEARDOWN):
            self.visited.append(keyword.name)


class ItemAdder(SuiteVisitor):
    test_to_add = 2
    test_started = False
    kw_added = False

    def start_test(self, test):
        if self.test_to_add > 0:
            test.parent.tests.create(name='Added by start_test')
            self.test_to_add -= 1
        self.test_started = True

    def end_test(self, test):
        if self.test_to_add > 0:
            test.parent.tests.create(name='Added by end_test')
            self.test_to_add -= 1
        self.test_started = False

    def start_keyword(self, keyword):
        if self.test_started and not self.kw_added:
            keyword.parent.body.create_keyword(kwname='Added by start_keyword')
            self.kw_added = True

    def end_keyword(self, keyword):
        if keyword.name == 'Added by start_keyword':
            keyword.parent.body.create_keyword(kwname='Added by end_keyword')


if __name__ == '__main__':
    unittest.main()



try:
    from collections.abc import Sequence, Mapping
except ImportError:
    from collections import Sequence, Mapping


__all__ = ['BROKEN_ITERABLE', 'BROKEN_SEQUENCE', 'BROKEN_MAPPING']


class BrokenIterable:

    def __iter__(self):
        yield 'x'
        raise ValueError(type(self).__name__)

    def __getitem__(self, item):
        return item

    def __len__(self):
        return 2


class BrokenSequence(BrokenIterable, Sequence):
    pass


class BrokenMapping(BrokenIterable, Mapping):
    pass



BROKEN_ITERABLE = BrokenIterable()
BROKEN_SEQUENCE = BrokenSequence()
BROKEN_MAPPING = BrokenMapping()



class DupeHybridKeywords:
    names = ['defined twice', 'DEFINED TWICE',
             'Embedded ${twice}', 'EMBEDDED ${ARG}',
             'Exact dupe is ok', 'Exact dupe is ok']

    def get_keyword_names(self):
        return self.names

    def __getattr__(self, name):
        if name not in self.names:
            raise AttributeError
        return lambda *args: None



import io
import sys
import unittest

from robot.utils import isatty
from robot.utils.asserts import assert_equal, assert_false, assert_raises


class TestIsATty(unittest.TestCase):

    def test_with_stdout_and_stderr(self):
        assert_equal(isatty(sys.__stdout__), sys.__stdout__.isatty())
        assert_equal(isatty(sys.__stderr__), sys.__stderr__.isatty())

    def test_with_io(self):
        with io.StringIO() as stream:
            assert_false(isatty(stream))
            wrapper = io.TextIOWrapper(stream, 'UTF-8')
            assert_false(isatty(wrapper))

    def test_with_detached_io_buffer(self):
        with io.StringIO() as stream:
            wrapper = io.TextIOWrapper(stream, 'UTF-8')
            wrapper.detach()
            assert_raises((ValueError, AttributeError), wrapper.isatty)
            assert_false(isatty(wrapper))

    def test_open_and_closed_file(self):
        with open(__file__) as file:
            assert_false(isatty(file))
        assert_false(isatty(file))


if __name__ == '__main__':
    unittest.main()



import inspect
import unittest
import re
from pathlib import Path

from robot.api import Language, Languages
from robot.conf.languages import En, Fi, PtBr, Th
from robot.errors import DataError
from robot.utils.asserts import (assert_equal, assert_not_equal, assert_raises,
                                 assert_raises_with_msg)


STANDARD_LANGUAGES = Language.__subclasses__()


class TestLanguage(unittest.TestCase):

    def test_one_part_code(self):
        assert_equal(Fi().code, 'fi')
        assert_equal(Fi.code, 'fi')

    def test_two_part_code(self):
        assert_equal(PtBr().code, 'pt-BR')
        assert_equal(PtBr.code, 'pt-BR')

    def test_name(self):
        assert_equal(Fi().name, 'Finnish')
        assert_equal(Fi.name, 'Finnish')
        assert_equal(PtBr().name, 'Brazilian Portuguese')
        assert_equal(PtBr.name, 'Brazilian Portuguese')

    def test_name_with_multiline_docstring(self):
        class X(Language):
            """Language Name

            Other lines are ignored.
            """
        assert_equal(X().name, 'Language Name')
        assert_equal(X.name, 'Language Name')

    def test_name_without_docstring(self):
        class X(Language):
            pass
        X.__doc__ = None
        assert_equal(X().name, '')
        assert_equal(X.name, '')

    def test_standard_languages_have_code_and_name(self):
        for cls in STANDARD_LANGUAGES:
            assert cls().code
            assert cls.code
            assert cls().name
            assert cls.name

    def test_standard_language_doc_formatting(self):
        added_in_rf60 = {'bg', 'bs', 'cs', 'de', 'en', 'es', 'fi', 'fr', 'hi',
                         'it', 'nl', 'pl', 'pt', 'pt-BR', 'ro', 'ru', 'sv',
                         'th', 'tr', 'uk', 'zh-CN', 'zh-TW'}
        for cls in STANDARD_LANGUAGES:
            doc = inspect.getdoc(cls)
            if cls.code in added_in_rf60:
                if doc != cls.name:
                    raise AssertionError(
                        f'Invalid docstring for {cls.name}. '
                        f'Expected only language name, got:\n{doc}'
                    )
            else:
                if not re.match(rf'{cls.name}\n\nNew in Robot Framework [\d.]+\.', doc):
                    raise AssertionError(
                        f'Invalid docstring for {cls.name}. '
                        f'Expected language name and "New in" note, got:\n{doc}'
                    )

    def test_code_and_name_of_Language_base_class_are_propertys(self):
        assert isinstance(Language.code, property)
        assert isinstance(Language.name, property)

    def test_eq(self):
        assert_equal(Fi(), Fi())
        assert_equal(Language.from_name('fi'), Fi())
        assert_not_equal(Fi(), PtBr())

    def test_hash(self):
        assert_equal(hash(Fi()), hash(Fi()))
        assert_equal({Fi(): 'value'}[Fi()], 'value')

    def test_subclasses_dont_have_wrong_attributes(self):
        for cls in Language.__subclasses__():
            for attr in dir(cls):
                if not hasattr(Language, attr):
                    raise AssertionError(f"Language class '{cls}' has attribute "
                                         f"'{attr}' not found on the base class.")

    def test_bdd_prefixes(self):
        class X(Language):
            given_prefixes = ['List', 'is', 'default']
            when_prefixes = {}
            but_prefixes = ('but', 'any', 'iterable', 'works')
        assert_equal(X().bdd_prefixes, {'List', 'is', 'default',
                                        'but', 'any', 'iterable', 'works'})


class TestLanguageFromName(unittest.TestCase):

    def test_code(self):
        assert isinstance(Language.from_name('fi'), Fi)
        assert isinstance(Language.from_name('FI'), Fi)

    def test_two_part_code(self):
        assert isinstance(Language.from_name('pt-BR'), PtBr)
        assert isinstance(Language.from_name('PTBR'), PtBr)

    def test_name(self):
        assert isinstance(Language.from_name('finnish'), Fi)
        assert isinstance(Language.from_name('Finnish'), Fi)

    def test_multi_part_name(self):
        assert isinstance(Language.from_name('Brazilian Portuguese'), PtBr)
        assert isinstance(Language.from_name('brazilianportuguese'), PtBr)

    def test_no_match(self):
        assert_raises_with_msg(ValueError, "No language with name 'no match' found.",
                               Language.from_name, 'no match')


class TestLanguages(unittest.TestCase):

    def test_init(self):
        assert_equal(list(Languages()), [En()])
        assert_equal(list(Languages('fi')), [Fi(), En()])
        assert_equal(list(Languages(['fi'])), [Fi(), En()])
        assert_equal(list(Languages(['fi', PtBr()])), [Fi(), PtBr(), En()])

    def test_init_without_default(self):
        assert_equal(list(Languages(add_english=False)), [])
        assert_equal(list(Languages('fi', add_english=False)), [Fi()])
        assert_equal(list(Languages(['fi'], add_english=False)), [Fi()])
        assert_equal(list(Languages(['fi', PtBr()], add_english=False)), [Fi(), PtBr()])

    def test_init_with_custom_language(self):
        path = Path(__file__).absolute().parent / 'orcish_languages.py'
        cwd = Path('.').absolute()
        for lang in (path, path.relative_to(cwd),
                     str(path), str(path.relative_to(cwd)),
                     [str(path)], [path]):
            langs = Languages(lang, add_english=False)
            assert_equal([("Orcish Loud", "or-CLOU"), ("Orcish Quiet", "or-CQUI")],
                         [(v.name, v.code) for v in langs])

    def test_reset(self):
        langs = Languages(['fi'])
        langs.reset()
        assert_equal(list(langs), [En()])
        langs.reset('fi')
        assert_equal(list(langs), [Fi(), En()])
        langs.reset(['fi', PtBr()])
        assert_equal(list(langs), [Fi(), PtBr(), En()])

    def test_reset_with_default(self):
        langs = Languages(['fi'])
        langs.reset(add_english=False)
        assert_equal(list(langs), [])
        langs.reset('fi', add_english=False)
        assert_equal(list(langs), [Fi()])
        langs.reset(['fi', PtBr()], add_english=False)
        assert_equal(list(langs), [Fi(), PtBr()])

    def test_duplicates_are_not_added(self):
        langs = Languages(['Finnish', 'en', Fi(), 'pt-br'])
        assert_equal(list(langs), [Fi(), En(), PtBr()])
        langs.add_language('en')
        assert_equal(list(langs), [Fi(), En(), PtBr()])
        langs.add_language('th')
        assert_equal(list(langs), [Fi(), En(), PtBr(), Th()])

    def test_add_language_using_custom_module(self):
        path = Path(__file__).absolute().parent / 'orcish_languages.py'
        cwd = Path('.').absolute()
        for lang in [path, path.relative_to(cwd), str(path), str(path.relative_to(cwd))]:
            langs = Languages(add_english=False)
            langs.add_language(lang)
            assert_equal([("Orcish Loud", "or-CLOU"), ("Orcish Quiet", "or-CQUI")],
                         [(v.name, v.code) for v in langs])

    def test_add_language_using_invalid_custom_module(self):
        error = assert_raises(DataError, Languages().add_language, 'non_existing_a23l4j')
        assert_equal(error.message.split(':')[0],
                     "No language with name 'non_existing_a23l4j' found. "
                     "Importing language file 'non_existing_a23l4j' failed")

    def test_add_language_using_invalid_custom_module_as_Path(self):
        invalid = Path('non_existing_a23l4j')
        assert_raises_with_msg(DataError,
                               f"Importing language file '{invalid.absolute()}' failed: "
                               f"File or directory does not exist.",
                               Languages().add_language, invalid)

    def test_add_language_using_Language_instance(self):
        languages = Languages(add_english=False)
        to_add = [Fi(), PtBr(), Th()]
        for lang in to_add:
            languages.add_language(lang)
        assert_equal(list(languages), to_add)


if __name__ == '__main__':
    unittest.main()



class DynamicKwOnlyArgsWithoutKwargs:

    def get_keyword_names(self):
        return ['No kwargs']

    def get_keyword_arguments(self, name):
        return ['*', 'kwo']

    def run_keyword(self, name, args):
        raise RuntimeError('Should not be executed!')



import sys
from remoteserver import DirectResultRemoteServer


class SpecialErrors:

    def continuable(self, message, traceback):
        return self._special_error(message, traceback, continuable=True)

    def fatal(self, message, traceback):
        return self._special_error(message, traceback,
                                   fatal='this wins', continuable=42)

    def _special_error(self, message, traceback, continuable=False, fatal=False):
        return {'status': 'FAIL', 'error': message, 'traceback': traceback,
                'continuable': continuable, 'fatal': fatal}


if __name__ == '__main__':
    DirectResultRemoteServer(SpecialErrors(), *sys.argv[1:])



import unittest

from robot.model import Message
from robot.result import Keyword
from robot.result.executionerrors import ExecutionErrors
from robot.utils.asserts import assert_equal, assert_raises


class TestMessage(unittest.TestCase):

    def test_slots(self):
        assert_raises(AttributeError, setattr, Message(), 'attr', 'value')

    def test_id_without_parent(self):
        assert_equal(Message().id, 'm1')

    def test_id_with_keyword_parent(self):
        kw = Keyword()
        assert_equal(kw.body.create_message().id, 'k1-m1')
        assert_equal(kw.body.create_message().id, 'k1-m2')
        assert_equal(kw.body.create_keyword().id, 'k1-k1')
        assert_equal(kw.body.create_message().id, 'k1-m3')
        assert_equal(kw.body.create_keyword().body.create_message().id, 'k1-k2-m1')

    def test_id_with_errors_parent(self):
        errors = ExecutionErrors()
        assert_equal(errors.messages.create().id, 'errors-m1')
        assert_equal(errors.messages.create().id, 'errors-m2')

    def test_id_when_item_not_in_parent(self):
        kw = Keyword()
        assert_equal(Message(parent=kw).id, 'k1-m1')
        assert_equal(kw.body.create_message().id, 'k1-m1')
        assert_equal(kw.body.create_message().id, 'k1-m2')
        assert_equal(Message(parent=kw).id, 'k1-m3')


class TestHtmlMessage(unittest.TestCase):

    def test_empty(self):
        assert_equal(Message().html_message, '')
        assert_equal(Message(html=True).html_message, '')

    def test_no_html(self):
        assert_equal(Message('Hello, Kitty!').html_message, 'Hello, Kitty!')
        assert_equal(Message('<b> & ftp://url').html_message,
                     '&lt;b&gt; &amp; <a href="ftp://url">ftp://url</a>')

    def test_html(self):
        assert_equal(Message('Hello, Kitty!', html=True).html_message, 'Hello, Kitty!')
        assert_equal(Message('<b> & ftp://x', html=True).html_message, '<b> & ftp://x')


class TestStringRepresentation(unittest.TestCase):

    def setUp(self):
        self.empty = Message()
        self.ascii = Message('Kekkonen', level='WARN')
        self.non_ascii = Message(u'hyv\xe4 nimi')

    def test_str(self):
        for tc, expected in [(self.empty, ''),
                             (self.ascii, 'Kekkonen'),
                             (self.non_ascii, u'hyv\xe4 nimi')]:
            assert_equal(str(tc), expected)

    def test_repr(self):
        for tc, expected in [(self.empty, "Message(message='', level='INFO')"),
                             (self.ascii, "Message(message='Kekkonen', level='WARN')"),
                             (self.non_ascii, u"Message(message=%r, level='INFO')"
                                              % u'hyv\xe4 nimi')]:
            assert_equal(repr(tc), 'robot.model.' + expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from pathlib import Path

from robot.utils.asserts import assert_equal
from robot.testdoc import JsonConverter, TestSuiteFactory

DATADIR = (Path(__file__).parent / '../../atest/testdata/misc').resolve()


def test_convert(item, **expected):
    for name in expected:
        assert_equal(item[name], expected[name])


class TestJsonConverter(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        suite = TestSuiteFactory(DATADIR, doc='My doc', metadata=['abc:123', '1:2'])
        cls.suite = JsonConverter(DATADIR / '../output.html').convert(suite)

    def test_suite(self):
        test_convert(self.suite,
                     source=str(DATADIR),
                     relativeSource='misc',
                     id='s1',
                     name='Misc',
                     fullName='Misc',
                     doc='<p>My doc</p>',
                     metadata=[('1', '<p>2</p>'), ('abc', '<p>123</p>')],
                     numberOfTests=192,
                     tests=[],
                     keywords=[])
        test_convert(self.suite['suites'][0],
                     source=str(DATADIR / 'dummy_lib_test.robot'),
                     relativeSource='misc/dummy_lib_test.robot',
                     id='s1-s1',
                     name='Dummy Lib Test',
                     fullName='Misc.Dummy Lib Test',
                     doc='',
                     metadata=[],
                     numberOfTests=1,
                     suites=[],
                     keywords=[])
        test_convert(self.suite['suites'][5]['suites'][1]['suites'][-1],
                     source=str(DATADIR / 'multiple_suites/02__sub.suite.1/second__.Sui.te.2..robot'),
                     relativeSource='misc/multiple_suites/02__sub.suite.1/second__.Sui.te.2..robot',
                     id='s1-s6-s2-s2',
                     name='.Sui.te.2.',
                     fullName='Misc.Multiple Suites.Sub.Suite.1..Sui.te.2.',
                     doc='',
                     metadata=[],
                     numberOfTests=12,
                     suites=[],
                     keywords=[])

    def test_multi_suite(self):
        data = TestSuiteFactory([DATADIR / 'normal.robot',
                                 DATADIR / 'pass_and_fail.robot'])
        suite = JsonConverter().convert(data)
        test_convert(suite,
                     source='',
                     relativeSource='',
                     id='s1',
                     name='Normal &amp; Pass And Fail',
                     fullName='Normal &amp; Pass And Fail',
                     doc='',
                     metadata=[],
                     numberOfTests=4,
                     keywords=[],
                     tests=[])
        test_convert(suite['suites'][0],
                     source=str(DATADIR / 'normal.robot'),
                     relativeSource='',
                     id='s1-s1',
                     name='Normal',
                     fullName='Normal &amp; Pass And Fail.Normal',
                     doc='<p>Normal test cases</p>',
                     metadata=[('Something', '<p>My Value</p>')],
                     numberOfTests=2)
        test_convert(suite['suites'][1],
                     source=str(DATADIR / 'pass_and_fail.robot'),
                     relativeSource='',
                     id='s1-s2',
                     name='Pass And Fail',
                     fullName='Normal &amp; Pass And Fail.Pass And Fail',
                     doc='<p>Some tests here</p>',
                     metadata=[],
                     numberOfTests=2)

    def test_test(self):
        test_convert(self.suite['suites'][0]['tests'][0],
                     id='s1-s1-t1',
                     name='Dummy Test',
                     fullName='Misc.Dummy Lib Test.Dummy Test',
                     doc='',
                     tags=[],
                     timeout='')
        test_convert(self.suite['suites'][4]['tests'][-7],
                     id='s1-s5-t5',
                     name='Fifth',
                     fullName='Misc.Many Tests.Fifth',
                     doc='',
                     tags=['d1', 'd2', 'f1'],
                     timeout='')
        test_convert(self.suite['suites'][-4]['tests'][0],
                     id='s1-s13-t1',
                     name='Default Test Timeout',
                     fullName='Misc.Timeouts.Default Test Timeout',
                     doc='<p>I have a timeout</p>',
                     tags=[],
                     timeout='1 minute 42 seconds')

    def test_timeout(self):
        suite = self.suite['suites'][-4]
        test_convert(suite['tests'][0],
                     name='Default Test Timeout',
                     timeout='1 minute 42 seconds')
        test_convert(suite['tests'][1],
                     name='Test Timeout With Variable',
                     timeout='${100}')
        test_convert(suite['tests'][2],
                     name='No Timeout',
                     timeout='')

    def test_keyword(self):
        test_convert(self.suite['suites'][0]['tests'][0]['keywords'][0],
                     name='dummykw',
                     arguments='',
                     type='KEYWORD')
        test_convert(self.suite['suites'][4]['tests'][-7]['keywords'][0],
                     name='Log',
                     arguments='Test 5',
                     type='KEYWORD')

    def test_suite_setup_and_teardown(self):
        test_convert(self.suite['suites'][4]['keywords'][0],
                     name='Log',
                     arguments='Setup',
                     type='SETUP')
        test_convert(self.suite['suites'][4]['keywords'][1],
                     name='No operation',
                     arguments='',
                     type='TEARDOWN')

    def test_test_setup_and_teardown(self):
        test_convert(self.suite['suites'][9]['tests'][0]['keywords'][0],
                     name='${TEST SETUP}',
                     arguments='',
                     type='SETUP')
        test_convert(self.suite['suites'][9]['tests'][0]['keywords'][2],
                     name='${TEST TEARDOWN}',
                     arguments='',
                     type='TEARDOWN')

    def test_for_loops(self):
        test_convert(self.suite['suites'][1]['tests'][0]['keywords'][0],
                     name='${pet} IN [ @{ANIMALS} ]',
                     arguments='',
                     type='FOR')
        test_convert(self.suite['suites'][1]['tests'][1]['keywords'][0],
                     name='${i} IN RANGE [ 10 ]',
                     arguments='',
                     type='FOR')

    def test_assign(self):
        test_convert(self.suite['suites'][6]['tests'][1]['keywords'][0],
                     name='${msg} = Evaluate',
                     arguments="u'Fran\\\\xe7ais'",
                     type='KEYWORD')


class TestFormattingAndEscaping(unittest.TestCase):
    suite = None

    def setUp(self):
        if not self.suite:
            suite = TestSuiteFactory(DATADIR / 'formatting_and_escaping.robot',
                                     name='<suite>', metadata=['CLI>:*bold*'])
            self.__class__.suite = JsonConverter().convert(suite)

    def test_suite_documentation(self):
        test_convert(self.suite,
                     doc='''\
<p>We have <i>formatting</i> and &lt;escaping&gt;.</p>
<table border="1">
<tr>
<td><b>Name</b></td>
<td><b>URL</b></td>
</tr>
<tr>
<td>Robot</td>
<td><a href="http://robotframework.org">http://robotframework.org</a></td>
</tr>
<tr>
<td>Custom</td>
<td><a href="http://robotframework.org">link</a></td>
</tr>
</table>''')

    def test_suite_metadata(self):
        test_convert(self.suite,
                     metadata=[('CLI&gt;', '<p><b>bold</b></p>'),
                               ('Escape', '<p>this is &lt;b&gt;not bold&lt;/b&gt;</p>'),
                               ('Format', '<p>this is <b>bold</b></p>')])

    def test_test_documentation(self):
        test_convert(self.suite['tests'][0],
                     doc='<p><b>I</b> can haz <i>formatting</i> &amp; &lt;escaping&gt;!!</p>'
                         '\n<ul>\n<li>list</li>\n<li>here</li>\n</ul>')

    def test_escaping(self):
        test_convert(self.suite, name='&lt;suite&gt;')
        test_convert(self.suite['tests'][1],
            name='&lt;Escaping&gt;',
            tags=['*not bold*', '&lt;b&gt;not bold either&lt;/b&gt;'],
            keywords=[{'type': 'KEYWORD',
                       'name': '&lt;blink&gt;NO&lt;/blink&gt;',
                       'arguments': '&lt;&amp;&gt;'}])


if __name__ == '__main__':
    unittest.main()



from collections import OrderedDict


def get_variables(*args):
    if args:
        return dict((args[i], args[i+1]) for i in range(0, len(args), 2))
    list_ = ['1', '2', 3]
    tuple_ = tuple(list_)
    dict_ = {'a': 1, 2: 'b', 'nested': {'key': 'value'}}
    ordered = OrderedDict((chr(o), o) for o in range(97, 107))
    open_file = open(__file__)
    closed_file = open(__file__)
    closed_file.close()
    return {'LIST__list': list_,
            'LIST__tuple': tuple_,
            'LIST__generator': (i for i in range(5)),
            'DICT__dict': dict_,
            'DICT__ordered': ordered,
            'scalar_list': list_,
            'scalar_tuple': tuple_,
            'scalar_generator': (i for i in range(5)),
            'scalar_dict': dict_,
            'failing_generator': failing_generator,
            'failing_dict': FailingDict({1: 2}),
            'open_file': open_file,
            'closed_file': closed_file}


def failing_generator():
    for i in [2, 1, 0]:
        yield 1/i


class FailingDict(dict):

    def __getattribute__(self, item):
        raise Exception('Bang')



"""Library for testing backwards compatibility.

Especially testing argument type information that has been changing after RF 4.
Examples are only using features compatible with all tested versions.
"""

from enum import Enum
from typing import Union
try:
    from typing_extensions import TypedDict
except ImportError:
    from typing import TypedDict


ROBOT_LIBRARY_VERSION = '1.0'


__all__ = ['simple', 'arguments', 'types', 'special_types', 'union']


class Color(Enum):
    """RGB colors."""
    RED = 'R'
    GREEN = 'G'
    BLUE = 'B'


class Size(TypedDict):
    """Some size."""
    width: int
    height: int


def simple():
    """Some doc.

    Tags: example
    """
    pass


def arguments(a, b=2, *c, d=4, e, **f):
    pass


def types(a: int, b: bool = True):
    pass


def special_types(a: Color, b: Size):
    pass


def union(a: Union[int, float]):
    pass



def one_argument(arg, /):
    return arg.upper()


def three_arguments(a, b, c, /):
    return '-'.join([a, b, c])


def with_normal(posonly, /, normal):
    return posonly + '-' + normal


def defaults(required, optional='default', /):
    return required + '-' + optional


def types(first: int, second: float, /):
    return first + second


def kwargs(x, /, **y):
    return '%s, %s' % (x, ', '.join('%s: %s' % item for item in y.items()))



from robot.api.deco import keyword


class UnknownType:
    pass


@keyword(types={'integer': int, 'boolean': bool, 'string': str})
def A_basics(integer, boolean, string):
    pass


@keyword(types={'integer': int, 'list_': list})
def B_with_defaults(integer=42, list_=None):
    pass


@keyword(types={'varargs': int, 'kwargs': bool})
def C_varags_and_kwargs(*varargs, **kwargs):
    pass


@keyword(types={'unknown': UnknownType, 'unrecognized': Ellipsis})
def D_unknown_types(unknown, unrecognized):
    pass


@keyword(types={'arg': 'One of the usages in PEP-3107',
                'varargs': 'But surely feels odd...'})
def E_non_type_annotations(arg, *varargs):
    pass


@keyword(types={'kwo': int, 'with_default': str})
def F_kw_only_args(*, kwo, with_default='value'):
    pass



import unittest
import warnings

from robot.utils.asserts import assert_equal, assert_true

from robot.running.runkwregister import _RunKeywordRegister as Register


class Lib:

    def method_without_arg(self):
        pass

    def method_with_one(self, name, *args):
        pass

    def method_with_default(self, one, two, three='default', *args):
        pass


def function_without_arg():
    pass


def function_with_one(name, *args):
    pass


def function_with_three(one, two, three, *args):
    pass


class TestRunKeywordRegister(unittest.TestCase):

    def setUp(self):
        self.reg = Register()

    def register_run_keyword(self, libname, keyword, args_to_process=None):
        self.reg.register_run_keyword(libname, keyword, args_to_process,
                                      deprecation_warning=False)

    def test_register_run_keyword_method_with_kw_name_and_arg_count(self):
        self._verify_reg('My Lib', 'myKeyword', 'My Keyword', 3, 3)

    def test_get_arg_count_with_non_existing_keyword(self):
        assert_equal(self.reg.get_args_to_process('My Lib', 'No Keyword'), -1)

    def test_get_arg_count_with_non_existing_library(self):
        self._verify_reg('My Lib', 'get_arg', 'Get Arg', 3, 3)
        assert_equal(self.reg.get_args_to_process('No Lib', 'Get Arg'), -1)

    def _verify_reg(self, lib_name, keyword, keyword_name, arg_count, given_count):
        self.register_run_keyword(lib_name, keyword, given_count)
        assert_equal(self.reg.get_args_to_process(lib_name, keyword_name), arg_count)

    def test_deprecation_warning(self):
        with warnings.catch_warnings(record=True) as w:
            self.reg.register_run_keyword('Library', 'Keyword', 0)
        [warning] = w
        assert_equal(
            str(warning.message),
            "The API to register run keyword variants and to disable variable resolving "
            "in keyword arguments will change in the future. For more information see "
            "https://github.com/robotframework/robotframework/issues/2190. "
            "Use with `deprecation_warning=False` to avoid this warning."
        )
        assert_true(issubclass(warning.category, UserWarning))


if __name__ == '__main__':
    unittest.main()



import unittest

from os.path import join

from robot import api, model, parsing, reporting, result, running
from robot.api import parsing as api_parsing

from robot.utils.asserts import assert_equal, assert_true


class TestExposedApi(unittest.TestCase):

    def test_execution_result(self):
        assert_equal(api.ExecutionResult, result.ExecutionResult)

    def test_test_suite(self):
        assert_equal(api.TestSuite, running.TestSuite)

    def test_result_writer(self):
        assert_equal(api.ResultWriter, reporting.ResultWriter)

    def test_visitors(self):
        assert_equal(api.SuiteVisitor, model.SuiteVisitor)
        assert_equal(api.ResultVisitor, result.ResultVisitor)

    def test_deprecated_parsing(self):
        assert_equal(api.get_model, parsing.get_model)
        assert_equal(api.get_resource_model, parsing.get_resource_model)
        assert_equal(api.get_tokens, parsing.get_tokens)
        assert_equal(api.get_resource_tokens, parsing.get_resource_tokens)
        assert_equal(api.Token, parsing.Token)

    def test_parsing_getters(self):
        assert_equal(api_parsing.get_model, parsing.get_model)
        assert_equal(api_parsing.get_resource_model, parsing.get_resource_model)
        assert_equal(api_parsing.get_tokens, parsing.get_tokens)
        assert_equal(api_parsing.get_resource_tokens, parsing.get_resource_tokens)

    def test_parsing_token(self):
        assert_equal(api_parsing.Token, parsing.Token)

    def test_parsing_model_statements(self):
        for cls in parsing.model.Statement.statement_handlers.values():
            assert_equal(getattr(api_parsing, cls.__name__), cls)
        assert_true(not hasattr(api_parsing, 'Statement'))

    def test_parsing_model_blocks(self):
        for name in ('File', 'SettingSection', 'VariableSection', 'TestCaseSection',
                     'KeywordSection', 'CommentSection', 'TestCase', 'Keyword', 'For',
                     'If', 'Try', 'While'):
            assert_equal(getattr(api_parsing, name), getattr(parsing.model, name))
        assert_true(not hasattr(api_parsing, 'Block'))

    def test_parsing_visitors(self):
        assert_equal(api_parsing.ModelVisitor, parsing.ModelVisitor)
        assert_equal(api_parsing.ModelTransformer, parsing.ModelTransformer)


class TestModelObjects(unittest.TestCase):
    """These model objects are part of the public API.

    They are only seldom needed directly and thus not exposed via the robot.api
    package. Tests just validate they are not removed accidentally.
    """

    def test_running_objects(self):
        assert_true(running.TestSuite)
        assert_true(running.TestCase)
        assert_true(running.Keyword)

    def test_result_objects(self):
        assert_true(result.TestSuite)
        assert_true(result.TestCase)
        assert_true(result.Keyword)


class TestTestSuiteBuilder(unittest.TestCase):
    # This list has paths like `/path/file.py/../file.robot` on purpose.
    # They don't work unless normalized.
    sources = [join(__file__, '../../../atest/testdata/misc', name)
               for name in ('pass_and_fail.robot', 'normal.robot')]

    def test_create_with_datasources_as_list(self):
        suite = api.TestSuiteBuilder().build(*self.sources)
        assert_equal(suite.name, 'Pass And Fail & Normal')

    def test_create_with_datasource_as_string(self):
        suite = api.TestSuiteBuilder().build(self.sources[0])
        assert_equal(suite.name, 'Pass And Fail')


if __name__ == '__main__':
    unittest.main()



import unittest
from io import BytesIO, StringIO

from robot.result import ExecutionResult
from robot.reporting.outputwriter import OutputWriter
from robot.utils import ET, ETSource, XmlWriter
from robot.utils.asserts import assert_equal

from test_resultbuilder import GOLDEN_XML, GOLDEN_XML_TWICE


class StreamXmlWriter(XmlWriter):

    def _order_attrs(self, attrs):
        return sorted(attrs.items())

    def _create_output(self, output):
        return output

    def close(self):
        pass


class TestableOutputWriter(OutputWriter):

    def _get_writer(self, output, rpa, generator):
        writer = StreamXmlWriter(output, write_empty=False)
        writer.start('robot')
        return writer


class TestResultSerializer(unittest.TestCase):

    def test_single_result_serialization(self):
        output = StringIO()
        writer = TestableOutputWriter(output)
        ExecutionResult(GOLDEN_XML).visit(writer)
        self._assert_xml_content(self._xml_lines(output.getvalue()),
                                 self._xml_lines(GOLDEN_XML))

    def _xml_lines(self, text):
        with ETSource(text) as source:
            tree = ET.parse(source)
        output = BytesIO()
        tree.write(output)
        return output.getvalue().splitlines()

    def _assert_xml_content(self, actual, expected):
        assert_equal(len(actual), len(expected))
        for index, (act, exp) in enumerate(list(zip(actual, expected))[2:]):
            assert_equal(act, exp.strip(), 'Different values on line %d' % index)

    def test_combining_results(self):
        output = StringIO()
        writer = TestableOutputWriter(output)
        ExecutionResult(GOLDEN_XML, GOLDEN_XML).visit(writer)
        self._assert_xml_content(self._xml_lines(output.getvalue()),
                                 self._xml_lines(GOLDEN_XML_TWICE))


if __name__ == '__main__':
    unittest.main()



KWS = {
    'Only tags in documentation': ('Tags: tag1, tag2', None),
    'Tags in addition to normal documentation': ('Normal doc\n\n...\n\nTags: tag', None),
    'Tags from get_keyword_tags': (None, ['t1', 't2', 't3']),
    'Tags both from doc and get_keyword_tags': ('Tags: 1, 2', ['4', '2', '3'])
}


class DynamicLibraryTags:
    get_keyword_tags_called = False

    def get_keyword_names(self):
        return list(KWS)

    def run_keyword(self, name, args, kwags):
        pass

    def get_keyword_documentation(self, name):
        if not self.get_keyword_tags_called:
            raise AssertionError("'get_keyword_tags' should be called before "
                                 "'get_keyword_documentation'")
        return KWS[name][0]

    def get_keyword_tags(self, name):
        self.get_keyword_tags_called = True
        return KWS[name][1]



from helper import pretty

def lib_mandatory_named_varargs_and_kwargs(a, b='default', *args, **kwargs):
    return pretty(a, b, *args, **kwargs)

def lib_kwargs(**kwargs):
    return pretty(**kwargs)

def lib_mandatory_named_and_kwargs(a, b=2, **kwargs):
    return pretty(a, b, **kwargs)

def lib_mandatory_named_and_varargs(a, b='default', *args):
    return pretty(a, b, *args)

def lib_mandatory_and_named(a, b='default'):
    return pretty(a, b)

def lib_mandatory_and_named_2(a, b='default', c='default'):
    return pretty(a, b, c)



SCALAR = 'Scalar from variable file from CLI'
SCALAR_WITH_ESCAPES = r'1 \ 2\\ ${inv}'
SCALAR_LIST = 'List variable value'.split()
LIST__LIST = SCALAR_LIST

PRIORITIES_1 = PRIORITIES_2 = 'Variable File from CLI'



import os.path
import functools

from robot.api.deco import library


__version__ = 'N/A'  # This should be ignored when version is parsed


class NameLibrary:    # Old-style class on purpose!
    handler_count = 10

    def simple1(self):
        """Simple 1"""

    def simple2___(self):
        """Simple 2"""

    def underscore_name(self):
        """Underscore Name"""

    def underscore_name2_(self):
        """Underscore Name2"""

    def un_der__sco_r__e_3(self):
        """Un Der Sco R E 3"""

    def MiXeD_CAPS(self):
        """MiXeD CAPS"""

    def camelCase(self):
        """Camel Case"""

    def camelCase2(self):
        """Camel Case 2"""

    def mixedCAPSCamel(self):
        """Mixed CAPS Camel"""

    def camelCase_and_underscores_doesNot_work(self):
        """CamelCase And Underscores DoesNot Work"""

    def _skipped1(self):
        """Starts with underscore"""

    skipped2 = "Not a function"


class DocLibrary:
    handler_count = 4

    def no_doc(self):
        pass

    no_doc.expected_doc = ''
    no_doc.expected_shortdoc = ''

    def one_line_doc(self):
        """One line doc"""

    one_line_doc.expected_doc = 'One line doc'
    one_line_doc.expected_shortdoc = 'One line doc'

    def multiline_doc(self):
        """First line is short doc.

        Full doc spans
        multiple lines.
        """

    multiline_doc.expected_doc = 'First line is short doc.\n\nFull doc spans\nmultiple lines.'
    multiline_doc.expected_shortdoc = 'First line is short doc.'

    def multiline_doc_with_split_short_doc(self):
        """Short doc can be split into
        multiple
        physical
        lines.

        This is documentation body and not included
        in short doc.

        Still body.
        """

    multiline_doc_with_split_short_doc.expected_doc = '''\
Short doc can be split into
multiple
physical
lines.

This is documentation body and not included
in short doc.

Still body.'''
    multiline_doc_with_split_short_doc.expected_shortdoc = '''\
Short doc can be split into
multiple
physical
lines.'''


class ArgInfoLibrary:
    handler_count = 13

    def no_args(self):
        """[], {}, None, None"""
        # Argument inspection had a bug when there was args on function body
        # so better keep some of them around here.
        a=b=c=1

    def required1(self, one):
        """['one'], {}, None, None"""

    def required2(self, one, two):
        """['one', 'two'], {}, None, None"""

    def required9(self, one, two, three, four, five, six, seven, eight, nine):
        """['one','two','three','four','five','six','seven','eight','nine'], \
           {}, None, None"""

    def default1(self, one=1):
        """['one'], {'one': 1}, None, None"""

    def default5(self, one='', two=None, three=3, four='huh', five=True):
        """['one', 'two', 'three', 'four', 'five'], \
           {'one': '', 'two': None, 'three': 3, 'four': 'huh', 'five': True}, \
           None, None"""

    def required1_default1(self, one, two=''):
        """['one', 'two'], {'two': ''}, None, None"""

    def required2_default3(self, one, two, three=3, four=4, five=5):
        """['one', 'two', 'three', 'four', 'five'], \
           {'three': 3, 'four': 4, 'five': 5}, None, None"""

    def varargs(self,*one):
        """[], {}, 'one', None"""

    def required2_varargs(self, one, two, *three):
        """['one', 'two'], {}, 'three', None"""

    def req4_def2_varargs(self, one, two, three, four, five=5, six=6, *seven):
        """['one', 'two', 'three', 'four', 'five', 'six'], \
           {'five': 5, 'six': 6}, 'seven', None"""

    def req2_def3_varargs_kwargs(self, three, four, five=5, six=6, seven=7, *eight, **nine):
        """['three', 'four', 'five', 'six', 'seven'], \
          {'five': 5, 'six': 6, 'seven': 7}, 'eight', 'nine'"""

    def varargs_kwargs(self,*one, **two):
        """[], {}, 'one', 'two'"""


class GetattrLibrary:
    handler_count = 3
    keyword_names = ['foo','bar','zap']

    def get_keyword_names(self):
        return self.keyword_names

    def __getattr__(self, name):
        def handler(*args):
            return name, args
        if name not in self.keyword_names:
            raise AttributeError
        return handler


class SynonymLibrary:
    handler_count = 3

    def handler(self):
        pass

    synonym_handler = handler
    another_synonym = handler


@library(auto_keywords=True)
class VersionLibrary:
    ROBOT_LIBRARY_VERSION = '0.1'
    ROBOT_LIBRARY_DOC_FORMAT = 'html'
    kw = lambda x:None


class VersionObjectLibrary:

    class _Version:
        def __init__(self, ver):
            self._ver = ver
        def __str__(self):
            return self._ver

    ROBOT_LIBRARY_VERSION = _Version('ver')
    kw = lambda x:None


class RecordingLibrary:
    ROBOT_LIBRARY_SCOPE = 'GLOBAL'

    def __init__(self):
        self.kw_accessed = 0
        self.kw_called = 0

    def kw(self):
        self.kw_called += 1

    def __getattribute__(self, name):
        if name == 'kw':
            self.kw_accessed += 1
        return object.__getattribute__(self, name)


class ArgDocDynamicLibrary:

    def __init__(self):
        kws = [('No Arg', [], None),
               ('One Arg', ['arg'], None),
               ('One or Two Args', ['arg', 'darg=dvalue'], None),
               ('Default as tuple', [('arg',), ('d1', False), ('d2', None)], None),
               ('Many Args', ['*args'], None),
               ('No Arg Spec', None, None),
               ('Multiline', None, 'Multiline\nshort doc!\n\nBody\nhere.')]
        self._keywords = dict((name, _KeywordInfo(name, argspec, doc))
                              for name, argspec, doc in kws)

    def get_keyword_names(self):
        return sorted(self._keywords)

    def run_keyword(self, name, args):
        print('*INFO* Executed keyword "%s" with arguments %s.' % (name, args))

    def get_keyword_documentation(self, name):
        return self._keywords[name].doc

    def get_keyword_arguments(self, name):
        return self._keywords[name].argspec


class ArgDocDynamicLibraryWithKwargsSupport(ArgDocDynamicLibrary):

    def __init__(self):
        ArgDocDynamicLibrary.__init__(self)
        for name, argspec in [('Kwargs', ['**kwargs']),
                              ('Varargs and Kwargs', ['*args', '**kwargs'])]:
            self._keywords[name] = _KeywordInfo(name, argspec)

    def run_keyword(self, name, args, kwargs={}):
        argstr = ' '.join([str(a) for a in args] +
                          ['%s:%s' % kv for kv in sorted(kwargs.items())])
        print('*INFO* Executed keyword %s with arguments %s' % (name, argstr))


class DynamicWithSource:
    path = os.path.normpath(os.path.dirname(__file__) + '/classes.py')
    keywords = {'only path': path,
                'path & lineno': path + ':42',
                'lineno only': ':6475',
                'invalid path': 'path validity is not validated',
                'path w/ colon': r'c:\temp\lib.py',
                'path w/ colon & lineno': r'c:\temp\lib.py:1234567890',
                'no source': None,
                'nön-äscii': 'hyvä esimerkki',
                'nön-äscii utf-8': b'\xe7\xa6\x8f:88',
                'invalid source': 666}

    def get_keyword_names(self):
        return list(self.keywords)

    def run_keyword(self, name, args, kws):
        pass

    def get_keyword_source(self, name):
        return self.keywords[name]


class _KeywordInfo:
    doc_template = 'Keyword documentation for %s'

    def __init__(self, name, argspec, doc=None):
        self.doc = doc or self.doc_template % name
        self.argspec = argspec


class InvalidGetDocDynamicLibrary(ArgDocDynamicLibrary):

    def get_keyword_documentation(self, name, invalid_arg):
        pass


class InvalidGetArgsDynamicLibrary(ArgDocDynamicLibrary):

    def get_keyword_arguments(self, name):
        1/0


class InvalidAttributeDynamicLibrary(ArgDocDynamicLibrary):
    get_keyword_documentation = True
    get_keyword_arguments = False


def noop(x):
    return x


def wraps(x):
    @functools.wraps(x)
    def wrapper(*a, **k):
        return x(*a, **k)
    return wrapper


@noop
@noop
@functools.total_ordering
class Decorated:

    @noop
    def no_wrapper(self):
        pass

    @noop
    @wraps
    @noop
    @wraps
    def wrapper(self):
        pass

    if hasattr(functools, 'lru_cache'):
        @functools.lru_cache()
        def external(self):
            pass

    no_def = lambda self: None

    def __eq__(self, other):
        return self is other

    def __lt__(self, other):
        return True


NoClassDefinition = type('NoClassDefinition', (), {})



from DynamicWithoutKwargs import DynamicWithoutKwargs


KEYWORDS = {
    'Kwargs': ['**kwargs'],
    'Args & Kwargs': ['a', 'b=default', ('c', 'xxx'), '**kwargs'],
    'Args, Varargs & Kwargs': ['a', 'b=default', '*varargs', '**kws'],
}


class DynamicWithKwargs(DynamicWithoutKwargs):

    def __init__(self):
        DynamicWithoutKwargs.__init__(self, **KEYWORDS)

    def run_keyword(self, name, args, kwargs):
        return self._pretty(*args, **kwargs)



class ExampleObject:
    
    def __init__(self, name='<noname>'):
        self.name = name

    def greet(self, name=None):
        if not name:
            return '%s says hi!' % self.name
        if name == 'FAIL':
            raise ValueError
        return '%s says hi to %s!' % (self.name, name)
    
    def __str__(self):
        return self.name
        
    def __repr__(self):
        return "'%s'" % self.name


OBJ = ExampleObject('dude')



def deprecated():
    """*DEPRECATED*"""


def deprecated_with_message():
    """*DEPRECATED for some good reason!* Yes it is. For sure."""


def no_deprecation_whatsoever():
    pass


def silent_deprecation():
    """*Deprecated* but not yet loudly.

    RF and Libdoc don't consider this being deprecated.
    """



class TOCWithInitsAndKeywords:
    """
    = First entry =

    TOC in somewhat strange place.

        %TOC%

    = Second =

             = 3 =

    %TOC% not replaced here
    """

    def __init__(self, arg=True):
        pass

    def keyword(self):
        """Tags: tag"""
        pass



import unittest

from robot.parsing import get_model, Token
from robot.parsing.model.statements import Break, Continue, Error, ReturnStatement

from parsing_test_utils import assert_model, RemoveNonDataTokensVisitor


def remove_non_data_nodes_and_assert(node, expected, data_only):
    if not data_only:
        RemoveNonDataTokensVisitor().visit(node)
    assert_model(node, expected)


class TestReturn(unittest.TestCase):

    def test_in_test_case_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    RETURN''', data_only=data_only)
                node = model.sections[0].body[0].body[0]
                expected = Error(
                    [Token(Token.ERROR, 'RETURN', 3, 4, 'RETURN is not allowed in this context.')],
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_test_case_body_inside_for(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    FOR    ${i}    IN    1    2
        RETURN
    END
        ''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = ReturnStatement(
                    [Token(Token.RETURN_STATEMENT, 'RETURN', 4, 8)],
                    errors=('RETURN can only be used inside a user keyword.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_test_case_body_inside_while(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    WHILE    True
        RETURN
    END
        ''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = ReturnStatement(
                    [Token(Token.RETURN_STATEMENT, 'RETURN', 4, 8)],
                    errors=('RETURN can only be used inside a user keyword.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_test_case_body_inside_if_else(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    IF    True
        RETURN
    ELSE IF    False
        RETURN
    ELSE
        RETURN
    END
        ''', data_only=data_only)
                ifroot = model.sections[0].body[0].body[0]
                node = ifroot.body[0]
                expected = ReturnStatement(
                    [Token(Token.RETURN_STATEMENT, 'RETURN', 4, 8)],
                    errors=('RETURN can only be used inside a user keyword.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)
                expected.tokens[0].lineno = 6
                remove_non_data_nodes_and_assert(ifroot.orelse.body[0], expected, data_only)
                expected.tokens[0].lineno = 8
                remove_non_data_nodes_and_assert(ifroot.orelse.orelse.body[0], expected, data_only)

    def test_in_test_case_body_inside_try_except(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    TRY
        RETURN
    EXCEPT
        RETURN
    ELSE
        RETURN
    FINALLY
        RETURN
    END
        ''', data_only=data_only)
                tryroot = model.sections[0].body[0].body[0]
                node = tryroot.body[0]
                expected = ReturnStatement(
                    [Token(Token.RETURN_STATEMENT, 'RETURN', 4, 8)],
                    errors=('RETURN can only be used inside a user keyword.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)
                expected.tokens[0].lineno = 6
                remove_non_data_nodes_and_assert(tryroot.next.body[0], expected, data_only)
                expected.tokens[0].lineno = 8
                remove_non_data_nodes_and_assert(tryroot.next.next.body[0], expected, data_only)
                expected.tokens[0].lineno = 10
                expected.errors += ('RETURN cannot be used in FINALLY branch.',)
                remove_non_data_nodes_and_assert(tryroot.next.next.next.body[0], expected, data_only)

    def test_in_finally_in_uk(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Keywords ***
Example
    TRY
        No operation
    EXCEPT
        No operation
    FINALLY
        RETURN
    END
        ''', data_only=data_only)
                node = model.sections[0].body[0].body[0].next.next.body[0]
                expected = ReturnStatement(
                    [Token(Token.RETURN_STATEMENT, 'RETURN', 8, 8)],
                    errors=('RETURN cannot be used in FINALLY branch.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_nested_finally_in_uk(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Keywords ***
Example
    IF    True
        TRY
            No operation
        EXCEPT
            No operation
        FINALLY
            RETURN
        END
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0].next.next.body[0]
                expected = ReturnStatement(
                    [Token(Token.RETURN_STATEMENT, 'RETURN', 9, 12)],
                    errors=('RETURN cannot be used in FINALLY branch.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)


class TestBreak(unittest.TestCase):

    def test_in_test_case_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    BREAK''', data_only=data_only)
                node = model.sections[0].body[0].body[0]
                expected = Error(
                    [Token(Token.ERROR, 'BREAK', 3, 4, 'BREAK is not allowed in this context.')],
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_if_test_case_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    IF    True
        BREAK
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = Break(
                    [Token(Token.BREAK, 'BREAK', 4, 8)],
                    errors=('BREAK can only be used inside a loop.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_try_test_case_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    TRY    
        BREAK
    EXCEPT
        no operation
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = Break(
                    [Token(Token.BREAK, 'BREAK', 4, 8)],
                    errors=('BREAK can only be used inside a loop.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_finally_inside_loop(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    WHILE    True
        TRY    
            Fail
        EXCEPT
            no operation
        FINALLY
           BREAK
        END     
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0].next.next.body[0]
                expected = Break(
                    [Token(Token.BREAK, 'BREAK', 9, 11)],
                    errors=('BREAK cannot be used in FINALLY branch.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_uk_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Keywords ***
Example
    BREAK''', data_only=data_only)
                node = model.sections[0].body[0].body[0]
                expected = Error(
                    [Token(Token.ERROR, 'BREAK', 3, 4, 'BREAK is not allowed in this context.')],
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_if_uk_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Keywords ***
Example
    IF    True
        BREAK
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = Break(
                    [Token(Token.BREAK, 'BREAK', 4, 8)],
                    errors=('BREAK can only be used inside a loop.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_try_uk_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Keywords ***
Example
    TRY    
        BREAK
    EXCEPT
        no operation
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = Break(
                    [Token(Token.BREAK, 'BREAK', 4, 8)],
                    errors=('BREAK can only be used inside a loop.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)


class TestContinue(unittest.TestCase):

    def test_in_test_case_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    CONTINUE''', data_only=data_only)
                node = model.sections[0].body[0].body[0]
                expected = Error(
                    [Token(Token.ERROR, 'CONTINUE', 3, 4, 'CONTINUE is not allowed in this context.')],
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_if_test_case_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    IF    True
        CONTINUE
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = Continue(
                    [Token(Token.CONTINUE, 'CONTINUE', 4, 8)],
                    errors=('CONTINUE can only be used inside a loop.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_try_test_case_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    TRY    
        CONTINUE
    EXCEPT
        no operation
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = Continue(
                    [Token(Token.CONTINUE, 'CONTINUE', 4, 8)],
                    errors=('CONTINUE can only be used inside a loop.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_finally_inside_loop(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Test Cases ***
Example
    WHILE    True
        TRY    
            Fail
        EXCEPT
            no operation
        FINALLY
           CONTINUE
        END     
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0].next.next.body[0]
                expected = Continue(
                    [Token(Token.CONTINUE, 'CONTINUE', 9, 11)],
                    errors=('CONTINUE cannot be used in FINALLY branch.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_uk_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Keywords ***
Example
    CONTINUE''', data_only=data_only)
                node = model.sections[0].body[0].body[0]
                expected = Error(
                    [Token(Token.ERROR, 'CONTINUE', 3, 4, 'CONTINUE is not allowed in this context.')],
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_if_uk_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Keywords ***
Example
    IF    True
        CONTINUE
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = Continue(
                    [Token(Token.CONTINUE, 'CONTINUE', 4, 8)],
                    errors=('CONTINUE can only be used inside a loop.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)

    def test_in_try_uk_body(self):
        for data_only in [True, False]:
            with self.subTest(data_only=data_only):
                model = get_model('''\
*** Keywords ***
Example
    TRY    
        CONTINUE
    EXCEPT
        no operation
    END''', data_only=data_only)
                node = model.sections[0].body[0].body[0].body[0]
                expected = Continue(
                    [Token(Token.CONTINUE, 'CONTINUE', 4, 8)],
                    errors=('CONTINUE can only be used inside a loop.',)
                )
                remove_non_data_nodes_and_assert(node, expected, data_only)


if __name__ == '__main__':
    unittest.main()



class TraceLogArgsLibrary:

    def only_mandatory(self, mand1, mand2):
        pass

    def mandatory_and_default(self, mand, default="default value"):
        pass

    def multiple_default_values(self, a=1, a2=2, a3=3, a4=4):
        pass

    def mandatory_and_varargs(self, mand, *varargs):
        pass

    def kwargs(self, **kwargs):
        pass

    def all_args(self, positional, *varargs, **kwargs):
        pass

    def return_object_with_non_ascii_repr(self):
        class NonAsciiRepr:
            def __repr__(self):
                return 'Hyv\xe4'
        return NonAsciiRepr()

    def return_object_with_invalid_repr(self):
        class InvalidRepr:
            def __repr__(self):
                raise ValueError
        return InvalidRepr()

    def embedded_arguments(self, *args):
        assert args == ('bar', 'Embedded Arguments')

    embedded_arguments.robot_name = 'Embedded Arguments "${a}" and "${b}"'



import unittest

from robot.utils.asserts import assert_equal
from robot.model import TestSuite
from robot.model.filter import Filter


class FilterBaseTest(unittest.TestCase):

    def _create_suite(self):
        self.s1 = TestSuite(name='s1')
        self.s21 = self.s1.suites.create(name='s21')
        self.s31 = self.s21.suites.create(name='s31')
        self.s31.tests.create(name='t1', tags=['t1', 's31'])
        self.s31.tests.create(name='t2', tags=['t2', 's31'])
        self.s31.tests.create(name='t3')
        self.s22 = self.s1.suites.create(name='s22')
        self.s22.tests.create(name='t1', tags=['t1', 's22', 'X'])

    def _test(self, filter, s31_tests, s22_tests):
        self._create_suite()
        self.s1.visit(filter)
        assert_equal([t.name for t in self.s31.tests], s31_tests)
        assert_equal([t.name for t in self.s22.tests], s22_tests)
        assert_equal(self.s1.test_count, len(s31_tests + s22_tests))


class TestFilterByIncludeTags(FilterBaseTest):

    def test_no_filtering(self):
        self._test(Filter(), ['t1', 't2', 't3'], ['t1'])
        self._test(Filter(include_tags=None), ['t1', 't2', 't3'], ['t1'])

    def test_empty_list_matches_none(self):
        self._test(Filter(include_tags=[]), [], [])

    def test_no_match(self):
        self._test(Filter(include_tags=['no', 'match']), [], [])

    def test_constant(self):
        self._test(Filter(include_tags=['t1']), ['t1'], ['t1'])

    def test_string(self):
        self._test(Filter(include_tags='t1'), ['t1'], ['t1'])

    def test_pattern(self):
        self._test(Filter(include_tags=['t*']), ['t1', 't2'], ['t1'])
        self._test(Filter(include_tags=['xxx', '?2', 's*2']), ['t2'], ['t1'])

    def test_normalization(self):
        self._test(Filter(include_tags=['T 1', '_T_2_']), ['t1', 't2'], ['t1'])

    def test_and_and_not(self):
        self._test(Filter(include_tags=['t1ANDs31']), ['t1'], [])
        self._test(Filter(include_tags=['?1ANDs*2ANDx']), [], ['t1'])
        self._test(Filter(include_tags=['t1ANDs*NOTx']), ['t1'], [])
        self._test(Filter(include_tags=['t1AND?1NOTs*ANDx']), ['t1'], [])


class TestFilterByExcludeTags(FilterBaseTest):

    def test_no_filtering(self):
        self._test(Filter(), ['t1', 't2', 't3'], ['t1'])
        self._test(Filter(exclude_tags=None), ['t1', 't2', 't3'], ['t1'])

    def test_empty_list_matches_none(self):
        self._test(Filter(exclude_tags=[]), ['t1', 't2', 't3'], ['t1'])

    def test_no_match(self):
        self._test(Filter(exclude_tags=['no', 'match']), ['t1', 't2', 't3'], ['t1'])

    def test_constant(self):
        self._test(Filter(exclude_tags=['t1']), ['t2', 't3'], [])

    def test_string(self):
        self._test(Filter(exclude_tags='t1'), ['t2', 't3'], [])

    def test_pattern(self):
        self._test(Filter(exclude_tags=['t*']), ['t3'], [])
        self._test(Filter(exclude_tags=['xxx', '?2', 's3*']), ['t3'], ['t1'])

    def test_normalization(self):
        self._test(Filter(exclude_tags=['T 1', '_T_2_']), ['t3'], [])

    def test_and_and_not(self):
        self._test(Filter(exclude_tags=['t1ANDs31']), ['t2', 't3'], ['t1'])
        self._test(Filter(exclude_tags=['?1ANDs*2ANDx']), ['t1', 't2', 't3'], [])
        self._test(Filter(exclude_tags=['t1ANDs*NOTx']), ['t2', 't3'], ['t1'])
        self._test(Filter(exclude_tags=['t1AND?1NOTs*ANDx']), ['t2', 't3'], ['t1'])


class TestFilterByTestName(FilterBaseTest):

    def test_no_filtering(self):
        self._test(Filter(), ['t1', 't2', 't3'], ['t1'])
        self._test(Filter(include_tests=None), ['t1', 't2', 't3'], ['t1'])

    def test_empty_list_matches_none(self):
        self._test(Filter(include_tests=[]), [], [])

    def test_no_match(self):
        self._test(Filter(include_tests=['no match']), [], [])

    def test_constant(self):
        self._test(Filter(include_tests=['t1']), ['t1'], ['t1'])
        self._test(Filter(include_tests=['t2', 'xxx']), ['t2'], [])

    def test_string(self):
        self._test(Filter(include_tests='t1'), ['t1'], ['t1'])

    def test_pattern(self):
        self._test(Filter(include_tests=['t*']), ['t1', 't2', 't3'], ['t1'])
        self._test(Filter(include_tests=['xxx', '*2', '?3']), ['t2', 't3'], [])

    def test_longname(self):
        self._test(Filter(include_tests=['s1.s21.s31.t3', 's1.s?2.*']), ['t3'], ['t1'])

    def test_normalization(self):
        self._test(Filter(include_tests=['T 1', '_T_2_']), ['t1', 't2'], ['t1'])


class TestFilterBySuiteName(FilterBaseTest):

    def test_no_filtering(self):
        self._test(Filter(), ['t1', 't2', 't3'], ['t1'])
        self._test(Filter(include_suites=None), ['t1', 't2', 't3'], ['t1'])

    def test_empty_list_matches_none(self):
        self._test(Filter(include_suites=[]), [], [])

    def test_no_match(self):
        self._test(Filter(include_suites=['no match']), [], [])

    def test_constant(self):
        self._test(Filter(include_suites=['s22']), [], ['t1'])
        self._test(Filter(include_suites=['s1', 'xxx']), ['t1', 't2', 't3'], ['t1'])

    def test_string(self):
        self._test(Filter(include_suites='s22'), [], ['t1'])

    def test_pattern(self):
        self._test(Filter(include_suites=['s3?']), ['t1', 't2', 't3'], [])

    def test_reuse_filter(self):
        filter = Filter(include_suites=['s22'])
        self._test(filter, [], ['t1'])
        self._test(filter, [], ['t1'])

    def test_parent_name(self):
        self._test(Filter(include_suites=['s1.s21.s31']), ['t1', 't2', 't3'], [])
        self._test(Filter(include_suites=['s2?.s31']), ['t1', 't2', 't3'], [])
        self._test(Filter(include_suites=['*.s22']), [], ['t1'])
        self._test(Filter(include_suites=['xxx.s22']), [], [])

    def test_normalization(self):
        self._test(Filter(include_suites=['_S 2 2_', 'xxx']), [], ['t1'])

    def test_with_other_filters(self):
        self._test(Filter(include_suites=['s21'], include_tests=['t1']), ['t1'], [])
        self._test(Filter(include_suites=['s22'], include_tags=['t*']), [], ['t1'])
        self._test(Filter(include_suites=['s21', 's22'], exclude_tags=['t?']), ['t3'], [])


class TestRemoveEmptySuitesDuringFilter(FilterBaseTest):

    def test_remove_empty_leaf_suite(self):
        self._test(Filter(include_tags='t2'), ['t2'], [])
        assert_equal(list(self.s1.suites), [self.s21])

    def test_remove_branch(self):
        self._test(Filter(include_suites='s22'), [], ['t1'])
        assert_equal(list(self.s1.suites), [self.s22])

    def test_remove_all(self):
        self._test(Filter(include_tests='none'), [], [])
        assert_equal(list(self.s1.suites), [])


if __name__ == '__main__':
    unittest.main()



from robot.errors import TimeoutError


class timeouting_listener:
    ROBOT_LISTENER_API_VERSION = 2
    timeout = False

    def start_keyword(self, name, info):
        self.timeout = name == 'BuiltIn.Log'

    def end_keyword(self, name, info):
        self.timeout = False

    def log_message(self, message):
        if self.timeout:
            self.timeout = False
            raise TimeoutError('Emulated timeout inside log_message')



from collections import UserDict
from collections.abc import Mapping


def get_variables(type):
    return {'dict': get_dict,
            'mydict': MyDict,
            'Mapping': get_MyMapping,
            'UserDict': get_UserDict,
            'MyUserDict': MyUserDict}[type]()


def get_dict():
    return {'from dict': 'This From Dict', 'from dict2': 2}


class MyDict(dict):

    def __init__(self):
        dict.__init__(self, from_my_dict='This From My Dict', from_my_dict2=2)


def get_MyMapping():
    data = {'from Mapping': 'This From Mapping', 'from Mapping2': 2}

    class MyMapping(Mapping):

        def __init__(self, data):
            self.data = data

        def __getitem__(self, item):
            return self.data[item]

        def __len__(self):
            return len(self.data)

        def __iter__(self):
            return iter(self.data)

    return MyMapping(data)


def get_UserDict():
    return UserDict({'from UserDict': 'This From UserDict', 'from UserDict2': 2})


class MyUserDict(UserDict):

    def __init__(self):
        UserDict.__init__(self, {'from MyUserDict': 'This From MyUserDict',
                                 'from MyUserDict2': 2})



import unittest
from robot.utils.asserts import (assert_equal, assert_false, assert_true,
                                 assert_raises, assert_raises_with_msg)

from robot.model.itemlist import ItemList


class Object:
    attr = 1

    def __init__(self, id=None):
        self.id = id

    def __eq__(self, other):
        return isinstance(other, Object) and self.id == other.id


class CustomItems(ItemList):
    pass


class TestItemLists(unittest.TestCase):

    def test_create_items(self):
        items = ItemList(str)
        item = items.create(object=1)
        assert_true(isinstance(item, str))
        assert_equal(item, '1')
        assert_equal(list(items), [item])

    def test_create_with_args_and_kwargs(self):
        class Item:
            def __init__(self, arg1, arg2):
                self.arg1 = arg1
                self.arg2 = arg2
        items = ItemList(Item)
        item = items.create('value 1', arg2='value 2')
        assert_equal(item.arg1, 'value 1')
        assert_equal(item.arg2, 'value 2')
        assert_equal(list(items), [item])

    def test_append_and_extend(self):
        items = ItemList(int)
        items.append(1)
        items.append(2)
        items.extend((3, 4))
        assert_equal(list(items), [1, 2, 3, 4])

    def test_extend_with_generator(self):
        items = ItemList(str)
        items.extend((c for c in 'Hello, world!'))
        assert_equal(list(items), list('Hello, world!'))

    def test_insert(self):
        items = ItemList(str)
        items.insert(0, 'a')
        items.insert(0, 'b')
        items.insert(3, 'c')
        items.insert(1, 'd')
        assert_equal(list(items), ['b', 'd', 'a', 'c'])

    def test_only_matching_types_can_be_added(self):
        assert_raises_with_msg(TypeError,
                               'Only integer objects accepted, got string.',
                               ItemList(int).append, 'not integer')
        assert_raises_with_msg(TypeError,
                               'Only integer objects accepted, got Object.',
                               ItemList(int).extend, [Object()])
        assert_raises_with_msg(TypeError,
                               'Only Object objects accepted, got integer.',
                               ItemList(Object).insert, 0, 42)

    def test_initial_items(self):
        assert_equal(list(ItemList(Object, items=[])), [])
        assert_equal(list(ItemList(int, items=(1, 2, 3))), [1, 2, 3])

    def test_common_attrs(self):
        item1 = Object()
        item2 = Object()
        parent = object()
        items = ItemList(Object, {'attr': 2, 'parent': parent}, [item1])
        items.append(item2)
        assert_true(item1.parent is parent)
        assert_equal(item1.attr, 2)
        assert_true(item2.parent is parent)
        assert_equal(item2.attr, 2)
        assert_equal(list(items), [item1, item2])

    def test_getitem(self):
        item1 = object()
        item2 = object()
        items = ItemList(object, items=[item1, item2])
        assert_true(items[0] is item1)
        assert_true(items[1] is item2)
        assert_true(items[-1] is item2)

    def test_getitem_slice(self):
        items = ItemList(int, items=range(10))
        sub = items[:5]
        assert_true(isinstance(sub, ItemList))
        assert_equal(list(sub), list(range(5)))
        assert_equal(list(items), list(range(10)))
        sub.append(5)
        assert_equal(list(sub), list(range(6)))
        assert_equal(list(items), list(range(10)))
        backwards = items[::-1]
        assert_true(isinstance(backwards, ItemList))
        assert_equal(list(backwards), list(reversed(items)))
        empty = items[100:]
        assert_true(isinstance(empty, ItemList))
        assert_equal(list(empty), [])

    def test_index(self):
        items = ItemList(str, items=('first', 'second'))
        assert_equal(items.index('first'), 0)
        assert_equal(items.index('second'), 1)
        assert_raises(ValueError, items.index, 'nonex')

    def test_index_with_start_and_stop(self):
        numbers = [0, 1, 2, 3, 2, 1, 0]
        items = ItemList(int, items=numbers)
        for num in sorted(set(numbers)):
            for start in range(len(numbers)):
                if num in numbers[start:]:
                    assert_equal(items.index(num, start),
                                 numbers.index(num, start))
                    for end in range(start, len(numbers)):
                        if num in numbers[start:end]:
                            assert_equal(items.index(num, start, end),
                                         numbers.index(num, start, end))

    def test_setitem(self):
        orig1, orig2 = Object(), Object()
        new1, new2 = Object(), Object()
        items = ItemList(Object, {'attr': 2}, [orig1, orig2])
        items[0] = new1
        assert_equal(list(items), [new1, orig2])
        assert_equal(new1.attr, 2)
        items[-1] = new2
        assert_equal(list(items), [new1, new2])
        assert_equal(new2.attr, 2)

    def test_setitem_slice(self):
        items = ItemList(int, items=range(10))
        items[:5] = []
        items[-2:] = [42]
        assert_equal(list(items), [5, 6, 7, 42])
        items = CustomItems(Object, {'a': 1}, [Object(i) for i in range(10)])
        items[1::3] = tuple(Object(c) for c in 'abc')
        assert_true(all(obj.a == 1 for obj in items))
        assert_equal([obj.id for obj in items],
                     [0, 'a', 2, 3, 'b', 5, 6, 'c', 8, 9])

    def test_setitem_slice_invalid_type(self):
        assert_raises_with_msg(TypeError,
                               'Only integer objects accepted, got float.',
                               ItemList(int).__setitem__, slice(0), [1, 1.1])

    def test_delitem(self):
        items = ItemList(str, items='abcde')
        del items[0]
        assert_equal(list(items), list('bcde'))
        del items[1]
        assert_equal(list(items), list('bde'))
        del items[-1]
        assert_equal(list(items), list('bd'))
        assert_raises(IndexError, items.__delitem__, 10)
        assert_equal(list(items), list('bd'))

    def test_delitem_slice(self):
        items = ItemList(str, items='abcde')
        del items[1:3]
        assert_equal(list(items), list('ade'))
        del items[2:]
        assert_equal(list(items), list('ad'))
        del items[10:]
        assert_equal(list(items), list('ad'))
        del items[:]
        assert_equal(list(items), [])

    def test_pop(self):
        items = ItemList(str, items='abcde')
        assert_equal(items.pop(), 'e')
        assert_equal(items.pop(0), 'a')
        assert_equal(items.pop(-2), 'c')
        assert_equal(list(items), ['b', 'd'])
        assert_raises(IndexError, items.pop, 7)
        assert_equal(list(items), ['b', 'd'])
        assert_raises(IndexError, ItemList(int).pop)

    def test_remove(self):
        items = ItemList(str, items='abcba')
        items.remove('c')
        assert_equal(list(items), list('abba'))
        items.remove('a')
        assert_equal(list(items), list('bba'))
        items.remove('b')
        items.remove('a')
        items.remove('b')
        assert_equal(list(items), list(''))
        assert_raises(ValueError, items.remove, 'nonex')

    def test_len(self):
        items = ItemList(object)
        assert_equal(len(items), 0)
        items.create()
        assert_equal(len(items), 1)

    def test_truth(self):
        assert_true(not ItemList(int))
        assert_true(ItemList(int, items=[1]))

    def test_contains(self):
        items = ItemList(str, items='x')
        assert_true('x' in items)
        assert_true('y' not in items)
        assert_false('x' not in items)
        assert_false('y' in items)

    def test_clear(self):
        items = ItemList(int, items=range(10))
        assert_equal(len(items), 10)
        items.clear()
        assert_equal(len(items), 0)

    def test_str(self):
        assert_equal(str(ItemList(int, items=[1, 2, 3, 4])), '[1, 2, 3, 4]')
        assert_equal(str(ItemList(str, items=['foo', 'bar'])), "['foo', 'bar']")
        assert_equal(str(ItemList(int, items=[1, 2, 3, 4])), '[1, 2, 3, 4]')
        assert_equal(str(ItemList(str, items=['hyvää', 'yötä'])), "['hyvää', 'yötä']")

    def test_repr(self):
        assert_equal(repr(ItemList(int, items=[1, 2, 3, 4])),
                     'ItemList(item_class=int, items=[1, 2, 3, 4])')
        assert_equal(repr(CustomItems(Object)),
                     'CustomItems(item_class=Object, items=[])')

    def test_iter(self):
        numbers = list(range(10))
        items = ItemList(int, items=numbers)
        assert_equal(list(items), numbers)
        assert_equal(tuple(items), tuple(numbers))
        for i, n in zip(items, numbers):
            assert_equal(i, n)

    def test_modifications_during_iter(self):
        chars = ItemList(str, items='abdx')
        for c in chars:
            if c == 'a':
                chars.pop()
            if c == 'b':
                chars.insert(2, 'c')
            if c == 'c':
                chars.append('e')
            assert_true(c in 'abcde', '%s was unexpected here!' % c)
        assert_equal(list(chars), list('abcde'))

    def test_count(self):
        obj1 = object()
        obj2 = object()
        objects = ItemList(object, items=[obj1, obj2, object(), obj2])
        assert_equal(objects.count(obj1), 1)
        assert_equal(objects.count(obj2), 2)
        assert_equal(objects.count(object()), 0)
        assert_equal(objects.count('whatever'), 0)

    def test_sort(self):
        chars = ItemList(str, items='asDfG')
        chars.sort()
        assert_equal(list(chars), ['D', 'G', 'a', 'f', 's'])
        chars.sort(key=str.lower)
        assert_equal(list(chars), ['a', 'D', 'f', 'G', 's'])
        chars.sort(reverse=True)
        assert_equal(list(chars), ['s', 'f', 'a', 'G', 'D'])

    def test_sorted(self):
        chars = ItemList(str, items='asdfg')
        assert_equal(sorted(chars), sorted('asdfg'))

    def test_reverse(self):
        chars = ItemList(str, items='asdfg')
        chars.reverse()
        assert_equal(list(chars), list(reversed('asdfg')))

    def test_reversed(self):
        chars = ItemList(str, items='asdfg')
        assert_equal(list(reversed(chars)), list(reversed('asdfg')))

    def test_modifications_during_reversed(self):
        chars = ItemList(str, items='yxdba')
        for c in reversed(chars):
            if c == 'a':
                chars.remove('x')
            if c == 'b':
                chars.insert(-2, 'c')
            if c == 'c':
                chars.pop(0)
            if c == 'd':
                chars.insert(0, 'e')
            assert_true(c in 'abcde', '%s was unexpected here!' % c)
        assert_equal(list(chars), list('edcba'))

    def test_comparisons(self):
        n123 = ItemList(int, items=[1, 2, 3])
        n123b = ItemList(int, items=[1, 2, 3])
        n1234 = ItemList(int, items=[1, 2, 3, 4])
        n124 = ItemList(int, items=[1, 2, 4])
        assert_true(n123 == n123b)
        assert_false(n123 != n123b)
        assert_true(n123 != n1234)
        assert_false(n123 == n1234)
        assert_true(n1234 > n123)
        assert_true(n1234 >= n123)
        assert_false(n1234 < n123)
        assert_false(n1234 <= n123)
        assert_true(n124 > n123)
        assert_true(n124 >= n123)
        assert_false(n124 < n123)
        assert_false(n124 <= n123)
        assert_true(n123 >= n123)
        assert_true(n123 <= n123)

    def test_compare_incompatible(self):
        assert_false(ItemList(int) == ItemList(str))
        assert_false(ItemList(int) == ItemList(int, {'a': 1}))
        assert_raises_with_msg(TypeError, 'Cannot order incompatible ItemLists.',
                               ItemList(int).__gt__, ItemList(str))
        assert_raises_with_msg(TypeError, 'Cannot order incompatible ItemLists.',
                               ItemList(int).__gt__, ItemList(int, {'a': 1}))

    def test_comparisons_with_other_objects(self):
        items = ItemList(int, items=[1, 2, 3])
        assert_false(items == 123)
        assert_false(items == [1, 2, 3])
        assert_false(items == (1, 2, 3))
        assert_true(items != 123)
        assert_true(items != [1, 2, 3])
        assert_true(items != (1, 2, 3))
        assert_raises_with_msg(TypeError, 'Cannot order ItemList and integer.',
                               items.__gt__, 1)
        assert_raises_with_msg(TypeError, 'Cannot order ItemList and list.',
                               items.__lt__, [1, 2, 3])
        assert_raises_with_msg(TypeError, 'Cannot order ItemList and tuple.',
                               items.__ge__, (1, 2, 3))

    def test_add(self):
        assert_equal(ItemList(int, items=[1, 2]) + ItemList(int, items=[3, 4]),
                     ItemList(int, items=[1, 2, 3, 4]))

    def test_add_incompatible(self):
        assert_raises_with_msg(TypeError,
                               'Cannot add ItemList and list.',
                               ItemList(int).__add__, [])
        assert_raises_with_msg(TypeError,
                               'Cannot add incompatible ItemLists.',
                               ItemList(int).__add__, ItemList(str))
        assert_raises_with_msg(TypeError,
                               'Cannot add incompatible ItemLists.',
                               ItemList(int).__add__, ItemList(int, {'a': 1}))

    def test_iadd(self):
        items = ItemList(int, items=[1, 2])
        items += (3, 4)
        items += [5]
        items += (i for i in (6, 7))
        items += ItemList(int, items=[8, 9])
        items += ItemList(int)
        assert_equal(items, ItemList(int, items=[1, 2, 3, 4, 5, 6, 7, 8, 9]))

    def test_iadd_incompatible(self):
        items = ItemList(int, items=[1, 2])
        assert_raises_with_msg(TypeError, 'Cannot add incompatible ItemLists.',
                               items.__iadd__, ItemList(str))
        assert_raises_with_msg(TypeError, 'Cannot add incompatible ItemLists.',
                               items.__iadd__, ItemList(int, {'a': 1}))

    def test_iadd_wrong_type(self):
        assert_raises_with_msg(TypeError,
                               'Only integer objects accepted, got string.',
                               ItemList(int).__iadd__, ['a', 'b', 'c'])

    def test_mul(self):
        assert_equal(ItemList(int, items=[1, 2, 3]) * 2,
                     ItemList(int, items=[1, 2, 3, 1, 2, 3]))
        assert_raises(TypeError, ItemList(int).__mul__, ItemList(int))

    def test_imul(self):
        items = ItemList(int, items=[1, 2])
        items *= 2
        items *= 1
        assert_equal(items, ItemList(int, items=[1, 2, 1, 2]))

    def test_rmul(self):
        assert_equal(2 * ItemList(int, items=[1, 2, 3]),
                     ItemList(int, items=[1, 2, 3, 1, 2, 3]))
        assert_raises(TypeError, ItemList(int).__rmul__, ItemList(int))

    def test_items_as_dicts_without_from_dict(self):
        items = ItemList(Object, items=[{'id': 1}, {}])
        items.append({'id': 3})
        assert_equal(items[0].id, 1)
        assert_equal(items[1].id, None)
        assert_equal(items[2].id, 3)

    def test_items_as_dicts_with_from_dict(self):
        class ObjectWithFromDict(Object):
            @classmethod
            def from_dict(cls, data):
                obj = cls()
                for name in data:
                    setattr(obj, name, data[name])
                return obj

        items = ItemList(ObjectWithFromDict, items=[{'id': 1, 'attr': 2}])
        items.extend([{}, {'new': 3}])
        assert_equal(items[0].id, 1)
        assert_equal(items[0].attr, 2)
        assert_equal(items[1].id, None)
        assert_equal(items[1].attr, 1)
        assert_equal(items[2].new, 3)

    def test_to_dicts_without_to_dict(self):
        items = ItemList(Object, items=[Object(1), Object(2)])
        dicts = items.to_dicts()
        assert_equal(dicts, [{'id': 1}, {'id': 2}])
        assert_equal(ItemList(Object, items=dicts), items)

    def test_to_dicts_with_to_dict(self):
        class ObjectWithToDict(Object):
            def to_dict(self):
                return {'id': self.id, 'x': 42}

        items = ItemList(ObjectWithToDict, items=[ObjectWithToDict(1)])
        assert_equal(items.to_dicts(), [{'id': 1, 'x': 42}])


if __name__ == '__main__':
    unittest.main()



def kw_only_arg(*, kwo):
    return kwo


def many_kw_only_args(*, first, second, third):
    return first + second + third


def kw_only_arg_with_default(*, kwo='default', another='another'):
    return '{}-{}'.format(kwo, another)


def mandatory_after_defaults(*, default1='xxx', mandatory, default2='zzz'):
    return '{}-{}-{}'.format(default1, mandatory, default2)


def kw_only_arg_with_annotation(*, kwo: str):
    return kwo


def kw_only_arg_with_annotation_and_default(*, kwo: str='default'):
    return kwo


def kw_only_arg_with_varargs(*varargs, kwo):
    return '-'.join(varargs + (kwo,))


def all_arg_types(pos_req, pos_def='pd', *varargs,
                  kwo_req, kwo_def='kd', **kwargs):
    varargs = list(varargs)
    kwargs = ['%s=%s' % item for item in sorted(kwargs.items())]
    return '-'.join([pos_req, pos_def] + varargs + [kwo_req, kwo_def] + kwargs)



import unittest

from robot.variables import (contains_variable,
                             is_variable, is_assign,
                             is_scalar_variable, is_scalar_assign,
                             is_list_variable, is_list_assign,
                             is_dict_variable, is_dict_assign,
                             search_variable)


SCALARS = ['${var}', '${  v A  R }']
LISTS = ['@{var}', '@{  v A  R }']
DICTS = ['&{var}', '&{  v A  R }']
NOKS = ['', 'nothing', '$not', '${not', '@not', '&{not', '${not}[oops',
        '%{not}', '*{not}', r'\${var}', r'\\\${var}', 42, None, ['${var}']]
NOK_ASSIGNS = NOKS + ['${${internal}}',
                      '@{${internal}}',
                      '&{${internal}}']


class TestIsVariable(unittest.TestCase):

    def test_is_variable(self):
        for ok in SCALARS + LISTS + DICTS:
            assert is_variable(ok)
            assert is_variable(ok + '[item]')
            assert search_variable(ok).is_variable()
            assert not is_variable(' ' + ok)
            assert not is_variable(ok + '=')
        for nok in NOKS:
            assert not is_variable(nok)
            assert not search_variable(nok, identifiers='$@&',
                                       ignore_errors=True).is_variable()

    def test_is_scalar_variable(self):
        for ok in SCALARS:
            assert is_scalar_variable(ok)
            assert is_scalar_variable(ok + '[item]')
            assert search_variable(ok).is_variable()
            assert not is_scalar_variable(' ' + ok)
            assert not is_scalar_variable(ok + '=')
        for nok in NOKS + LISTS + DICTS:
            assert not is_scalar_variable(nok)
            assert not search_variable(nok, ignore_errors=True).is_scalar_variable()

    def test_is_list_variable(self):
        for ok in LISTS:
            assert is_list_variable(ok)
            assert search_variable(ok).is_list_variable()
            assert is_list_variable(ok + '[item]')
            assert not is_list_variable(' ' + ok)
            assert not is_list_variable(ok + '=')
        for nok in NOKS + SCALARS + DICTS:
            assert not is_list_variable(nok)
            assert not search_variable(nok, ignore_errors=True).is_list_variable()

    def test_is_dict_variable(self):
        for ok in DICTS:
            assert is_dict_variable(ok)
            assert search_variable(ok).is_dict_variable()
            assert is_dict_variable(ok + '[item]')
            assert not is_dict_variable(' ' + ok)
            assert not is_dict_variable(ok + '=')
        for nok in NOKS + SCALARS + LISTS:
            assert not is_dict_variable(nok)
            assert not search_variable(nok, ignore_errors=True).is_dict_variable()

    def test_contains_variable(self):
        for ok in SCALARS + LISTS + DICTS + [r'\${no ${yes}!']:
            assert contains_variable(ok)
            assert contains_variable(ok + '[item]')
            assert contains_variable('hello %s world' % ok)
            assert contains_variable('hello %s[item] world' % ok)
            assert contains_variable(' ' + ok)
            assert contains_variable(r'\\' + ok)
            assert contains_variable(ok + '=')
            assert contains_variable(ok + ok)
        for nok in NOKS:
            assert not contains_variable(nok)


class TestIsAssign(unittest.TestCase):

    def test_is_assign(self):
        for ok in SCALARS + LISTS + DICTS:
            assert is_assign(ok)
            assert search_variable(ok).is_assign()
            assert is_assign(ok + '=', allow_assign_mark=True)
            assert is_assign(ok + ' =', allow_assign_mark=True)
            assert not is_assign(' ' + ok)
        for ok in SCALARS + LISTS + DICTS:
            assert is_assign(ok + '[item]' + '[ i t e m ]' + '[${item}]', allow_items=True)
            assert not is_assign(ok + '[item]' + '[ i t e m ]' + '[${item}]')
            assert is_assign(ok + '[item]' + '[ i t e m ]' + '[${item}]', allow_items=True)
            assert not is_assign(ok + '[item]' + '[ i t e m ]' + '[${item}]')
        for nok in NOK_ASSIGNS:
            assert not is_assign(nok)
            assert not search_variable(nok, ignore_errors=True).is_assign()

    def test_is_scalar_assign(self):
        for ok in SCALARS:
            assert is_scalar_assign(ok)
            assert search_variable(ok).is_scalar_assign()
            assert is_scalar_assign(ok + '=', allow_assign_mark=True)
            assert is_scalar_assign(ok + ' =', allow_assign_mark=True)
            assert is_scalar_assign(ok + '[item]', allow_items=True)
            assert is_scalar_assign(ok + '[item1][item2]', allow_items=True)
            assert not is_scalar_assign(ok + '[item]')
            assert not is_scalar_assign(ok + '[item1][item2]')
            assert not is_scalar_assign(' ' + ok)
        for nok in NOK_ASSIGNS + LISTS + DICTS:
            assert not is_scalar_assign(nok)
            assert not search_variable(nok, ignore_errors=True).is_scalar_assign()

    def test_is_list_assign(self):
        for ok in LISTS:
            assert is_list_assign(ok)
            assert search_variable(ok).is_list_assign()
            assert is_list_assign(ok + '=', allow_assign_mark=True)
            assert is_list_assign(ok + ' =', allow_assign_mark=True)
            assert is_list_assign(ok + '[item]', allow_items=True)
            assert is_list_assign(ok + '[item1][item2]', allow_items=True)
            assert not is_list_assign(ok + '[item]')
            assert not is_list_assign(ok + '[item1][item2]')
            assert not is_list_assign(' ' + ok)
        for nok in NOK_ASSIGNS + SCALARS + DICTS:
            assert not is_list_assign(nok)
            assert not search_variable(nok, ignore_errors=True).is_list_assign()

    def test_is_dict_assign(self):
        for ok in DICTS:
            assert is_dict_assign(ok)
            assert search_variable(ok).is_dict_assign()
            assert is_dict_assign(ok + '=', allow_assign_mark=True)
            assert is_dict_assign(ok + ' =', allow_assign_mark=True)
            assert is_dict_assign(ok + '[item]', allow_items=True)
            assert is_dict_assign(ok + '[item1][item2]', allow_items=True)
            assert not is_dict_assign(ok + '[item]')
            assert not is_dict_assign(ok + '[item1][item2]')
            assert not is_dict_assign(' ' + ok)
        for nok in NOK_ASSIGNS + SCALARS + LISTS:
            assert not is_dict_assign(nok)
            assert not search_variable(nok, ignore_errors=True).is_dict_assign()


if __name__ == '__main__':
    unittest.main()



from robot.api.deco import keyword, library


@library
class LibraryDecorator:

    def not_keyword(self):
        raise RuntimeError('Should not be executed!')

    @keyword
    def decorated_method_is_keyword(self):
        print('Decorated methods are keywords.')



import unittest

from robot.model import For, If, IfBranch, TestCase, Try, TryBranch, While
from robot.utils.asserts import assert_equal


IF = If.IF
ELSE_IF = If.ELSE_IF
ELSE = If.ELSE
TRY = Try.TRY
EXCEPT = Try.EXCEPT
FINALLY = Try.FINALLY


class TestFor(unittest.TestCase):

    def test_string_reprs(self):
        for for_, exp_str, exp_repr in [
            (For(),
             'FOR    IN',
             "For(variables=(), flavor='IN', values=())"),
            (For(('${x}',), 'IN RANGE', ('10',)),
             'FOR    ${x}    IN RANGE    10',
             "For(variables=('${x}',), flavor='IN RANGE', values=('10',))"),
            (For(('${x}', '${y}'), 'IN ENUMERATE', ('a', 'b')),
             'FOR    ${x}    ${y}    IN ENUMERATE    a    b',
             "For(variables=('${x}', '${y}'), flavor='IN ENUMERATE', values=('a', 'b'))"),
            (For(['${x}'], 'IN ENUMERATE', ['@{stuff}'], start='1'),
             'FOR    ${x}    IN ENUMERATE    @{stuff}    start=1',
             "For(variables=('${x}',), flavor='IN ENUMERATE', values=('@{stuff}',), start='1')"),
            (For(('${x}', '${y}'), 'IN ZIP', ('${xs}', '${ys}'), mode='LONGEST', fill='-'),
             'FOR    ${x}    ${y}    IN ZIP    ${xs}    ${ys}    mode=LONGEST    fill=-',
             "For(variables=('${x}', '${y}'), flavor='IN ZIP', values=('${xs}', '${ys}'), mode='LONGEST', fill='-')"),
            (For(['${ü}'], 'IN', ['föö']),
             'FOR    ${ü}    IN    föö',
             "For(variables=('${ü}',), flavor='IN', values=('föö',))")
        ]:
            assert_equal(str(for_), exp_str)
            assert_equal(repr(for_), 'robot.model.' + exp_repr)


class TestWhile(unittest.TestCase):

    def test_string_reprs(self):
        for while_, exp_str, exp_repr in [
            (While(),
             'WHILE',
             "While(condition=None)"),
            (While('$x', limit='100'),
             'WHILE    $x    limit=100',
             "While(condition='$x', limit='100')")
        ]:
            assert_equal(str(while_), exp_str)
            assert_equal(repr(while_), 'robot.model.' + exp_repr)


class TestIf(unittest.TestCase):

    def test_type(self):
        assert_equal(IfBranch().type, IF)
        assert_equal(IfBranch(type=ELSE).type, ELSE)
        assert_equal(IfBranch(type=ELSE_IF).type, ELSE_IF)

    def test_type_with_nested_if(self):
        branch = IfBranch()
        branch.body.create_if()
        assert_equal(branch.body[0].body.create_branch().type, IF)
        assert_equal(branch.body[0].body.create_branch(ELSE_IF).type, ELSE_IF)
        assert_equal(branch.body[0].body.create_branch(ELSE).type, ELSE)

    def test_root_id(self):
        assert_equal(If().id, None)
        assert_equal(TestCase().body.create_if().id, None)

    def test_branch_id_without_parent(self):
        assert_equal(IfBranch().id, 'k1')

    def test_branch_id_with_only_root(self):
        root = If()
        assert_equal(root.body.create_branch().id, 'k1')
        assert_equal(root.body.create_branch().id, 'k2')

    def test_branch_id_with_only_root_when_branch_not_in_root(self):
        assert_equal(IfBranch(parent=If()).id, 'k1')

    def test_branch_id_with_real_parent(self):
        root = TestCase().body.create_if()
        assert_equal(root.body.create_branch().id, 't1-k1')
        assert_equal(root.body.create_branch().id, 't1-k2')

    def test_branch_id_when_parent_has_setup(self):
        tc = TestCase()
        assert_equal(tc.setup.config(name='X').id, 't1-k1')
        assert_equal(tc.body.create_keyword().id, 't1-k2')
        assert_equal(tc.body.create_if().body.create_branch().id, 't1-k3')
        assert_equal(tc.body.create_keyword().id, 't1-k4')
        assert_equal(tc.body.create_if().body.create_branch().id, 't1-k5')

    def test_string_reprs(self):
        for if_, exp_str, exp_repr in [
            (IfBranch(),
             'IF    None',
             "IfBranch(type='IF', condition=None)"),
            (IfBranch(condition='$x > 1'),
             'IF    $x > 1',
             "IfBranch(type='IF', condition='$x > 1')"),
            (IfBranch(ELSE_IF, condition='$x > 2'),
             'ELSE IF    $x > 2',
             "IfBranch(type='ELSE IF', condition='$x > 2')"),
            (IfBranch(ELSE),
             'ELSE',
             "IfBranch(type='ELSE', condition=None)"),
            (IfBranch(condition=u'$x == "\xe4iti"'),
             u'IF    $x == "\xe4iti"',
             u"IfBranch(type='IF', condition=%r)" % u'$x == "\xe4iti"'),
        ]:
            assert_equal(str(if_), exp_str)
            assert_equal(repr(if_), 'robot.model.' + exp_repr)


class TestTry(unittest.TestCase):

    def test_type(self):
        assert_equal(TryBranch().type, TRY)
        assert_equal(TryBranch(type=EXCEPT).type, EXCEPT)
        assert_equal(TryBranch(type=ELSE).type, ELSE)
        assert_equal(TryBranch(type=FINALLY).type, FINALLY)

    def test_type_with_nested_Try(self):
        branch = TryBranch()
        branch.body.create_try()
        assert_equal(branch.body[0].body.create_branch().type, TRY)
        assert_equal(branch.body[0].body.create_branch(type=EXCEPT).type, EXCEPT)
        assert_equal(branch.body[0].body.create_branch(type=ELSE).type, ELSE)
        assert_equal(branch.body[0].body.create_branch(type=FINALLY).type, FINALLY)

    def test_root_id(self):
        assert_equal(Try().id, None)
        assert_equal(TestCase().body.create_try().id, None)

    def test_branch_id_without_parent(self):
        assert_equal(TryBranch().id, 'k1')

    def test_branch_id_with_only_root(self):
        root = Try()
        assert_equal(root.body.create_branch().id, 'k1')
        assert_equal(root.body.create_branch().id, 'k2')

    def test_branch_id_with_only_root_when_branch_not_in_root(self):
        assert_equal(TryBranch(parent=Try()).id, 'k1')

    def test_branch_id_with_real_parent(self):
        root = TestCase().body.create_try()
        assert_equal(root.body.create_branch().id, 't1-k1')
        assert_equal(root.body.create_branch().id, 't1-k2')

    def test_branch_id_when_parent_has_setup(self):
        tc = TestCase()
        assert_equal(tc.setup.config(name='X').id, 't1-k1')
        assert_equal(tc.body.create_keyword().id, 't1-k2')
        assert_equal(tc.body.create_try().body.create_branch().id, 't1-k3')
        assert_equal(tc.body.create_keyword().id, 't1-k4')
        assert_equal(tc.body.create_try().body.create_branch().id, 't1-k5')

    def test_string_reprs(self):
        for try_, exp_str, exp_repr in [
            (TryBranch(),
             'TRY',
             "TryBranch(type='TRY')"),
            (TryBranch(EXCEPT),
             'EXCEPT',
             "TryBranch(type='EXCEPT')"),
            (TryBranch(EXCEPT, ('Message',)),
             'EXCEPT    Message',
             "TryBranch(type='EXCEPT', patterns=('Message',))"),
            (TryBranch(EXCEPT, ('M', 'S', 'G', 'S')),
             'EXCEPT    M    S    G    S',
             "TryBranch(type='EXCEPT', patterns=('M', 'S', 'G', 'S'))"),
            (TryBranch(EXCEPT, (), None, '${x}'),
             'EXCEPT    AS    ${x}',
             "TryBranch(type='EXCEPT', variable='${x}')"),
            (TryBranch(EXCEPT, ('Message',), 'glob', '${x}'),
             'EXCEPT    Message    type=glob    AS    ${x}',
             "TryBranch(type='EXCEPT', patterns=('Message',), pattern_type='glob', variable='${x}')"),
            (TryBranch(ELSE),
             'ELSE',
             "TryBranch(type='ELSE')"),
            (TryBranch(FINALLY),
             'FINALLY',
             "TryBranch(type='FINALLY')"),
        ]:
            assert_equal(str(try_), exp_str)
            assert_equal(repr(try_), 'robot.model.' + exp_repr)


if __name__ == '__main__':
    unittest.main()



from robot.utils import NormalizedDict
from robot.libraries.BuiltIn import BuiltIn

BUILTIN = BuiltIn()

KEYWORDS = NormalizedDict({'add_keyword': ('name', '*args'),
                           'remove_keyword': ('name',),
                           'reload_self': (),
                           'original 1': ('arg',),
                           'original 2': ('arg',),
                           'original 3': ('arg',)})

class Reloadable:

    def get_keyword_names(self):
        return list(KEYWORDS)

    def get_keyword_arguments(self, name):
        return KEYWORDS[name]

    def get_keyword_documentation(self, name):
        return 'Doc for %s with args %s' % (name, ', '.join(KEYWORDS[name]))

    def run_keyword(self, name, args):
        print("Running keyword '%s' with arguments %s." % (name, args))
        assert name in KEYWORDS
        if name == 'add_keyword':
            KEYWORDS[args[0]] = args[1:]
        elif name == 'remove_keyword':
            KEYWORDS.pop(args[0])
        elif name == 'reload_self':
            BUILTIN.reload_library(self)
        return name




VARIABLES = dict(exp_return_value=b'ty\xf6paikka',
                 exp_return_msg='ty\\xf6paikka',
                 exp_error_msg="b'hyv\\xe4'",
                 exp_log_msg="b'\\xe4iti'",
                 exp_log_multiline_msg="b'\\xe4iti\\nis\\xe4'")


def get_variables():
    return VARIABLES.copy()



class LenLibrary:
    """Library with default zero __len__.

    Example:

    >>> l = LenLibrary()
    >>> assert not l
    >>> l.set_length(1)
    >>> assert l
    """
    def __init__(self):
        self._length = 0

    def __len__(self):
        return self._length

    def set_length(self, length):
        self._length = int(length)



import asyncio

from robot.libraries.BuiltIn import BuiltIn


class Hanger:

    def __init__(self) -> None:
        self.task = None
        self.ticks = []

    async def start_async_process(self):
        while True:
            self.ticks.append('tick')
            await asyncio.sleep(0.01)


class AsyncLib:

    async def basic_async_test(self):
        await asyncio.sleep(0.1)
        return 'Got it'

    def async_with_run_inside(self):
        async def inner():
            await asyncio.sleep(0.1)
            return 'Works'
        return asyncio.run(inner())

    async def can_use_gather(self):
        tasks = [asyncio.sleep(0.1) for _ in range(5)]
        await asyncio.gather(*tasks)

    async def create_hanger(self):
        hanger = Hanger()
        hanger.task = asyncio.create_task(hanger.start_async_process())
        return hanger

    async def stop_task_from_hanger(self, hanger):
        hanger.task.cancel()

    async def run_keyword_using_builtin(self):
        return await BuiltIn().run_keyword("Basic Async Test")

    async def create_task_with_loop(self):
        loop = asyncio.get_event_loop()
        task = loop.create_task(self.basic_async_test())
        return await task



import signal
import unittest
from threading import Thread

from robot.output import LOGGER
from robot.output.loggerhelper import AbstractLogger
from robot.utils.asserts import assert_equal

from robot.running.signalhandler import _StopSignalMonitor


LOGGER.unregister_console_logger()


def assert_signal_handler_equal(signum, expected):
    sig = signal.getsignal(signum)
    assert_equal(sig, expected)


class LoggerStub(AbstractLogger):
    def __init__(self):
        AbstractLogger.__init__(self)
        self.messages = []
    def message(self, msg):
        self.messages.append(msg)


class TestSignalHandlerRegisteringFailures(unittest.TestCase):

    def setUp(self):
        self.logger = LoggerStub()
        LOGGER._message_cache = []
        LOGGER.register_logger(self.logger)
        self._orig_signal = signal.signal

    def tearDown(self):
        LOGGER.unregister_logger(self.logger)
        signal.signal = self._orig_signal

    def test_error_messages(self):
        def raise_value_error(signum, handler):
            raise ValueError("Got signal %d" % signum)
        signal.signal = raise_value_error
        _StopSignalMonitor().__enter__()
        assert_equal(len(self.logger.messages), 2)
        self._verify_warning(self.logger.messages[0], 'INT',
                             'Got signal %d' % signal.SIGINT)
        self._verify_warning(self.logger.messages[1], 'TERM',
                             'Got signal %d' % signal.SIGTERM)

    def _verify_warning(self, msg, signame, err):
        ctrlc = 'or with Ctrl-C ' if signame == 'INT' else ''
        assert_equal(msg.message,
                     'Registering signal %s failed. Stopping execution '
                     'gracefully with this signal %sis not possible. '
                     'Original error was: %s' % (signame, ctrlc, err))
        assert_equal(msg.level, 'WARN')

    def test_failure_but_no_warning_when_not_in_main_thread(self):
        t = Thread(target=_StopSignalMonitor().__enter__)
        t.start()
        t.join()
        assert_equal(len(self.logger.messages), 0)


class TestRestoringOriginalHandlers(unittest.TestCase):

    def get_int(self):
        return signal.getsignal(signal.SIGINT)

    def get_term(self):
        return signal.getsignal(signal.SIGTERM)

    def setUp(self):
        self.orig_int = self.get_int()
        self.orig_term = self.get_term()

    def tearDown(self):
        signal.signal(signal.SIGINT, self.orig_int)
        signal.signal(signal.SIGTERM, self.orig_term)

    def test_restore_when_no_failures(self):
        with _StopSignalMonitor() as monitor:
            assert_equal(self.get_int(), monitor)
            assert_equal(self.get_term(), monitor)
        assert_signal_handler_equal(signal.SIGINT, self.orig_int)
        assert_signal_handler_equal(signal.SIGTERM, self.orig_term)

    def test_restore_when_failure(self):
        try:
            with _StopSignalMonitor() as monitor:
                assert_equal(self.get_int(), monitor)
                assert_equal(self.get_term(), monitor)
                raise ZeroDivisionError
        except ZeroDivisionError:
            pass
        else:
            raise AssertionError
        assert_signal_handler_equal(signal.SIGINT, self.orig_int)
        assert_signal_handler_equal(signal.SIGTERM, self.orig_term)

    def test_registered_outside_python(self):
        """
        If a signal isn't registered within Python, signal.getsignal() returns None.
        This tests to make sure _StopSignalMonitor.__exit__ can handle that.
        """
        with _StopSignalMonitor() as monitor:
            assert_equal(self.get_int(), monitor)
            assert_equal(self.get_term(), monitor)
            monitor._orig_sigint = None
            monitor._orig_sigterm = None
        assert_equal(self.get_int(), signal.SIG_DFL)
        assert_equal(self.get_term(), signal.SIG_DFL)


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.result.executionerrors import ExecutionErrors, Message
from robot.utils.asserts import assert_equal


class TestExecutionErrors(unittest.TestCase):

    def test_str_without_messages(self):
        assert_equal(str(ExecutionErrors()), 'No execution errors')

    def test_str_with_one_message(self):
        assert_equal(str(ExecutionErrors([Message('Only one')])),
                     'Execution error: Only one')

    def test_str_with_multiple_messages(self):
        assert_equal(str(ExecutionErrors([Message('1st'), Message('2nd')])),
                     'Execution errors:\n- 1st\n- 2nd')


if __name__ == '__main__':
    unittest.main()



import threading
import logging
import time

from robot.api import logger



def log_using_robot_api_in_thread():
    threading.Timer(0.1, log_using_robot_api).start()

def log_using_robot_api():
    for i in range(100):
        logger.info(str(i))
        time.sleep(0.01)

def log_using_logging_module_in_thread():
    threading.Timer(0.1, log_using_logging_module).start()

def log_using_logging_module():
    for i in range(100):
        logging.info(str(i))
        time.sleep(0.01)



import json
import os
import pprint
import shlex
from pathlib import Path
from subprocess import run, PIPE, STDOUT

from jsonschema import Draft202012Validator
from xmlschema import XMLSchema

from robot.api import logger
from robot.utils import SYSTEM_ENCODING
from robot.running.arguments import ArgInfo


ROOT = Path(__file__).absolute().parent.parent.parent.parent


class LibDocLib:

    def __init__(self, interpreter=None):
        self.interpreter = interpreter
        self.xml_schema = XMLSchema(str(ROOT/'doc/schema/libdoc.xsd'))
        with open(ROOT/'doc/schema/libdoc.json') as f:
            self.json_schema = Draft202012Validator(json.load(f))

    @property
    def libdoc(self):
        return self.interpreter.libdoc

    def run_libdoc(self, args):
        cmd = self.libdoc + self._split_args(args)
        cmd[-1] = cmd[-1].replace('/', os.sep)
        logger.info(' '.join(cmd))
        result = run(cmd, cwd=ROOT/'src', stdout=PIPE, stderr=STDOUT,
                     encoding=SYSTEM_ENCODING, timeout=120, universal_newlines=True)
        logger.info(result.stdout)
        return result.stdout

    def _split_args(self, args):
        lexer = shlex.shlex(args, posix=True)
        lexer.escape = ''
        lexer.whitespace_split = True
        return list(lexer)

    def get_libdoc_model_from_html(self, path):
        with open(path, encoding='UTF-8') as html_file:
            model_string = self._find_model(html_file)
        model = json.loads(model_string)
        logger.info(pprint.pformat(model))
        return model

    def _find_model(self, html_file):
        for line in html_file:
            if line.startswith('libdoc = '):
                return line.split('=', 1)[1].strip(' \n;')
        raise RuntimeError('No model found from HTML')

    def validate_xml_spec(self, path):
        self.xml_schema.validate(path)

    def validate_json_spec(self, path):
        with open(path) as f:
            self.json_schema.validate(json.load(f))

    def get_repr_from_arg_model(self, model):
        return str(ArgInfo(kind=model['kind'],
                           name=model['name'],
                           type=model['type'] or ArgInfo.NOTSET,
                           default=model['default'] or ArgInfo.NOTSET))

    def get_repr_from_json_arg_model(self, model):
        return str(ArgInfo(kind=model['kind'],
                           name=model['name'],
                           type=model['type'] or ArgInfo.NOTSET,
                           default=model['defaultValue'] or ArgInfo.NOTSET))



def get_variables(name, value='default value'):
    if name == 'FAIL':
        1/0
    varz = { name: value,
             'ANOTHER_SCALAR': 'Variable from CLI var file with get_variables',
             'LIST__ANOTHER_LIST': ['List variable from CLI var file',
                                    'with get_variables'] }
    for name in 'PRIORITIES_1', 'PRIORITIES_2', 'PRIORITIES_2B':
        varz[name] = 'Second Variable File from CLI'
    return varz



import os

OUTFILE = open(os.path.join(os.getenv('TEMPDIR'), 'listener_attrs.txt'), 'w')
START = 'doc starttime '
END = START + 'endtime elapsedtime status '
SUITE = 'id longname metadata source tests suites totaltests '
TEST = 'id longname tags template originalname source lineno '
KW = 'kwname libname args assign tags type lineno source status '
KW_TYPES = {'FOR': 'variables flavor values',
            'WHILE': 'condition limit on_limit_message',
            'IF': 'condition',
            'ELSE IF': 'condition',
            'EXCEPT': 'patterns pattern_type variable',
            'RETURN': 'values'}
FOR_FLAVOR_EXTRA = {'IN ENUMERATE': ' start',
                    'IN ZIP': ' mode fill'}
EXPECTED_TYPES = {'tags': [str],
                  'args': [str],
                  'assign': [str],
                  'metadata': {str: str},
                  'tests': [str],
                  'suites': [str],
                  'totaltests': int,
                  'elapsedtime': int,
                  'lineno': (int, type(None)),
                  'source': (str, type(None)),
                  'variables': (dict, list),
                  'flavor': str,
                  'values': (list, dict),
                  'condition': str,
                  'limit': (str, type(None)),
                  'on_limit_message': (str, type(None)),
                  'patterns': (str, list),
                  'pattern_type': (str, type(None)),
                  'variable': (str, type(None))}


def verify_attrs(method_name, attrs, names):
    names = set(names.split())
    OUTFILE.write(method_name + '\n')
    if len(names) != len(attrs):
        OUTFILE.write(f'FAILED: wrong number of attributes\n')
        OUTFILE.write(f'Expected: {sorted(names)}\n')
        OUTFILE.write(f'Actual:   {sorted(attrs)}\n')
        return
    for name in names:
        value = attrs[name]
        exp_type = EXPECTED_TYPES.get(name, str)
        if isinstance(exp_type, list):
            verify_attr(name, value, list)
            for index, item in enumerate(value):
                verify_attr('%s[%s]' % (name, index), item, exp_type[0])
        elif isinstance(exp_type, dict):
            verify_attr(name, value, dict)
            key_type, value_type = dict(exp_type).popitem()
            for key, value in value.items():
                verify_attr('%s[%s] (key)' % (name, key), key, key_type)
                verify_attr('%s[%s] (value)' % (name, key), value, value_type)
        else:
            verify_attr(name, value, exp_type)


def verify_attr(name, value, exp_type):
    if isinstance(value, exp_type):
        OUTFILE.write('passed | %s: %s\n' % (name, format_value(value)))
    else:
        OUTFILE.write('FAILED | %s: %r, Expected: %s, Actual: %s\n'
                      % (name, value, exp_type, type(value)))


def format_value(value):
    if isinstance(value, str):
        return value
    if isinstance(value, int):
        return str(value)
    if isinstance(value, list):
        return '[%s]' % ', '.join(format_value(item) for item in value)
    if isinstance(value, dict):
        return '{%s}' % ', '.join('%s: %s' % (format_value(k), format_value(v))
                                  for k, v in value.items())
    if value is None:
        return 'None'
    return 'FAILED! Invalid argument type %s.' % type(value)


def verify_name(name, kwname=None, libname=None, **ignored):
    if libname:
        if name != '%s.%s' % (libname, kwname):
            OUTFILE.write("FAILED | KW NAME: '%s' != '%s.%s'\n" % (name, libname, kwname))
    else:
        if name != kwname:
            OUTFILE.write("FAILED | KW NAME: '%s' != '%s'\n" % (name, kwname))
        if libname != '':
            OUTFILE.write("FAILED | LIB NAME: '%s' != ''\n" % libname)


class VerifyAttributes:
    ROBOT_LISTENER_API_VERSION = '2'

    def __init__(self):
        self._keyword_stack = []

    def start_suite(self, name, attrs):
        verify_attrs('START SUITE', attrs, START + SUITE)

    def end_suite(self, name, attrs):
        verify_attrs('END SUITE', attrs, END + SUITE + 'statistics message')

    def start_test(self, name, attrs):
        verify_attrs('START TEST', attrs, START + TEST)

    def end_test(self, name, attrs):
        verify_attrs('END TEST', attrs, END + TEST + 'message')

    def start_keyword(self, name, attrs):
        type_ = attrs['type']
        extra = KW_TYPES.get(type_, '')
        if type_ == 'ITERATION' and self._keyword_stack[-1] == 'FOR':
            extra += ' variables'
        if type_ == 'FOR':
            extra += FOR_FLAVOR_EXTRA.get(attrs['flavor'], '')
        verify_attrs('START ' + type_, attrs, START + KW + extra)
        verify_name(name, **attrs)
        self._keyword_stack.append(type_)

    def end_keyword(self, name, attrs):
        self._keyword_stack.pop()
        type_ = attrs['type']
        extra = KW_TYPES.get(type_, '')
        if type_ == 'ITERATION' and self._keyword_stack[-1] == 'FOR':
            extra += ' variables'
        if type_ == 'FOR':
            extra += FOR_FLAVOR_EXTRA.get(attrs['flavor'], '')
        verify_attrs('END ' + type_, attrs, END + KW + extra)
        verify_name(name, **attrs)

    def close(self):
        OUTFILE.close()



import unittest
from collections import OrderedDict

from robot.utils import DotDict
from robot.utils.asserts import (assert_equal, assert_false, assert_not_equal,
                                 assert_raises, assert_true)


class TestDotDict(unittest.TestCase):

    def setUp(self):
        self.dd = DotDict([('z', 1), (2, 'y'), ('x', 3)])

    def test_init(self):
        assert_true(DotDict() == DotDict({}) == DotDict([]))
        assert_true(DotDict(a=1) == DotDict({'a': 1}) == DotDict([('a', 1)]))
        assert_true(DotDict({'a': 1}, b=2) ==
                    DotDict({'a': 1, 'b': 2}) ==
                    DotDict([('a', 1), ('b', 2)]))
        assert_raises(TypeError, DotDict, None)

    def test_get(self):
        assert_equal(self.dd[2], 'y')
        assert_equal(self.dd.x, 3)
        assert_raises(KeyError, self.dd.__getitem__, 'nonex')
        assert_raises(AttributeError, self.dd.__getattr__, 'nonex')

    def test_equality(self):
        assert_true(self.dd == self.dd)
        assert_false(self.dd != self.dd)
        assert_true(self.dd == DotDict(self.dd))
        assert_false(self.dd != DotDict(self.dd))
        assert_false(self.dd == DotDict())
        assert_true(self.dd != DotDict())

    def test_equality_with_normal_dict(self):
        assert_equal(self.dd, {'z': 1, 2: 'y', 'x': 3})

    def test_hash(self):
        assert_raises(TypeError, hash, self.dd)

    def test_set(self):
        self.dd.x = 42
        self.dd.new = 43
        self.dd[2] = 44
        self.dd['n2'] = 45
        assert_equal(self.dd, {'z': 1, 2: 44, 'x': 42, 'new': 43, 'n2': 45})

    def test_del(self):
        del self.dd.x
        del self.dd[2]
        self.dd.pop('z')
        assert_equal(self.dd, {})
        assert_raises(KeyError, self.dd.__delitem__, 'nonex')
        assert_raises(AttributeError, self.dd.__delattr__, 'nonex')

    def test_same_str_and_repr_format_as_with_normal_dict(self):
        D = {'foo': 'bar', '"\'': '"\'', '\n': '\r', 1: 2, (): {}, True: False}
        for d in {}, {'a': 1}, D:
            for formatter in str, repr:
                result = formatter(DotDict(d))
                assert_equal(eval(result, {}), d)

    def test_is_ordered(self):
        assert_equal(list(self.dd), ['z', 2, 'x'])
        self.dd.z = 'new value'
        self.dd.a_new_item = 'last'
        self.dd.pop('x')
        assert_equal(list(self.dd.items()),
                     [('z', 'new value'), (2, 'y'), ('a_new_item', 'last')])
        self.dd.x = 'last'
        assert_equal(list(self.dd.items()),
                     [('z', 'new value'), (2, 'y'), ('a_new_item', 'last'), ('x', 'last')])

    def test_order_does_not_affect_equality(self):
        d = dict(a=1, b=2, c=3, d=4, e=5, f=6, g=7)
        od1 = OrderedDict(sorted(d.items()))
        od2 = OrderedDict(reversed(list(od1.items())))
        dd1 = DotDict(sorted(d.items()))
        dd2 = DotDict(reversed(list(dd1.items())))
        for d1, d2 in [(dd1, dd2), (dd1, d), (dd2, d), (dd1, od1), (dd2, od2)]:
            assert_equal(d1, d2)
            assert_equal(d2, d1)
        for d1, d2 in [(dd1, od2), (dd2, od1)]:
            assert_equal(d1, d2)
            assert_equal(d2, d1)
        assert_not_equal(od1, od2)


class TestNestedDotDict(unittest.TestCase):

    def test_nested_dicts_are_converted_to_dotdicts_at_init(self):
        leaf = {'key': 'value'}
        d = DotDict({'nested': leaf, 'deeper': {'nesting': leaf}}, nested2=leaf)
        assert_equal(d.nested.key, 'value')
        assert_equal(d.deeper.nesting.key, 'value')
        assert_equal(d.nested2.key, 'value')

    def test_dicts_inside_lists_are_converted(self):
        leaf = {'key': 'value'}
        d = DotDict(list=[leaf, leaf, [leaf]], deeper=[leaf, {'deeper': leaf}])
        assert_equal(d.list[0].key, 'value')
        assert_equal(d.list[1].key, 'value')
        assert_equal(d.list[2][0].key, 'value')
        assert_equal(d.deeper[0].key, 'value')
        assert_equal(d.deeper[1].deeper.key, 'value')

    def test_other_list_like_items_are_not_touched(self):
        value = ({'key': 'value'}, [{}])
        d = DotDict(key=value)
        assert_equal(d.key[0]['key'], 'value')
        assert_false(hasattr(d.key[0], 'key'))
        assert_true(isinstance(d.key[0], dict))
        assert_true(isinstance(d.key[1][0], dict))

    def test_items_inserted_outside_init_are_not_converted(self):
        d = DotDict()
        d['dict'] = {'key': 'value'}
        d['list'] = [{}]
        assert_equal(d.dict['key'], 'value')
        assert_false(hasattr(d.dict, 'key'))
        assert_true(isinstance(d.dict, dict))
        assert_true(isinstance(d.list[0], dict))

    def test_dotdicts_are_not_recreated(self):
        value = DotDict(key=1)
        d = DotDict(key=value)
        assert_true(d.key is value)
        assert_equal(d.key.key, 1)

    def test_lists_are_not_recreated(self):
        value = [{'key': 1}]
        d = DotDict(key=value)
        assert_true(d.key is value)
        assert_equal(d.key[0].key, 1)


if __name__ == '__main__':
    unittest.main()



from robot.api.deco import keyword, library


@library(version='3.2b1', scope='GLOBAL', doc_format='HTML')
class LibraryDecorator:
    ROBOT_LIBRARY_VERSION = 'overridden'

    @keyword
    def kw(self):
        pass

    def not_kw(self):
        pass



from pathlib import Path

from robot.api import TestSuite
from robot.api.interfaces import Parser, TestDefaults

import custom


class CustomParser(Parser):

    def __init__(self, extension='custom', parse=True, init=False, fail=False,
                 bad_return=False):
        self.extension = extension.split(',') if extension else None
        if not parse:
            self.parse = None
        if init:
            self.extension.append('init')
        else:
            self.parse_init = None
        self.fail = fail
        self.bad_return = bad_return

    def parse(self, source: Path, defaults: TestDefaults) -> TestSuite:
        if self.fail:
            raise TypeError('Ooops!')
        if self.bad_return:
            return 'bad'
        suite = custom.parse(source)
        suite.name = TestSuite.name_from_source(source, self.extension)
        for test in suite.tests:
            defaults.set_to(test)
        return suite

    def parse_init(self, source: Path, defaults: TestDefaults) -> TestSuite:
        if self.fail:
            raise TypeError('Ooops in init!')
        if self.bad_return:
            return 42
        defaults.tags = ['tag from init']
        defaults.setup = {'name': 'Log', 'args': ['setup from init']}
        defaults.teardown = {'name': 'Log', 'args': ['teardown from init']}
        defaults.timeout = '42s'
        return TestSuite(name='📁', source=source.parent, metadata={'Parser': 'Custom'})



from robot.api.deco import library

import sys


class listener:
    ROBOT_LISTENER_API_VERSION = 2

    def start_test(self, name, attrs):
        self._stderr("START TEST")

    def end_test(self, name, attrs):
        self._stderr("END TEST")

    def log_message(self, msg):
        self._stderr("MESSAGE %s" % msg['message'])

    def close(self):
        self._stderr("CLOSE")

    def _stderr(self, msg):
        sys.__stderr__.write("%s\n" % msg)


@library(scope='TEST CASE', listener=listener())
class empty_listenerlibrary:
    pass



import unittest

from robot.utils.asserts import assert_true, assert_raises
from robot.utils import Sortable


class MySortable(Sortable):

    def __init__(self, sort_key=NotImplemented):
        self._sort_key = sort_key


class TestSortable(unittest.TestCase):

    def setUp(self):
        self.a = MySortable('a')
        self.a2 = MySortable('a')
        self.b = MySortable('b')

    def test_eq(self):
        assert_true(self.a == self.a2)
        assert_true(not self.a == self.b)
        assert_true(not self.a == 1)

    def test_ne(self):
        assert_true(self.a != self.b)
        assert_true(not self.a != self.a2)
        assert_true(self.a != 1)

    def test_lt(self):
        assert_true(self.a < self.b)
        assert_true(not self.a < self.a2)
        assert_raises(TypeError, lambda: self.a < 1)

    def test_gt(self):
        assert_true(self.b > self.a)
        assert_true(not self.a > self.a2)
        assert_raises(TypeError, lambda: self.a > 1)

    def test_le(self):
        assert_true(self.a <= self.a)
        assert_true(self.a <= self.b)
        assert_true(not self.b <= self.a)
        assert_raises(TypeError, lambda: self.a <= 1)

    def test_ge(self):
        assert_true(self.a >= self.a)
        assert_true(self.b >= self.a)
        assert_true(not self.a >= self.b)
        assert_raises(TypeError, lambda: self.a >= 1)


if __name__ == '__main__':
    unittest.main()



import unittest
import time

from robot.utils.asserts import assert_equal
from robot.utils import format_time

from robot.output.stdoutlogsplitter import StdoutLogSplitter as Splitter


class TestOutputSplitter(unittest.TestCase):

    def test_empty_output_should_result_in_empty_messages_list(self):
        splitter = Splitter('')
        assert_equal(list(splitter), [])

    def test_plain_output_should_have_info_level(self):
        splitter = Splitter('this is message\nin many\nlines.')
        self._verify_message(splitter, 'this is message\nin many\nlines.')
        assert_equal(len(list(splitter)), 1)

    def test_leading_and_trailing_space_should_be_stripped(self):
        splitter = Splitter('\t  \n My message    \t\r\n')
        self._verify_message(splitter, 'My message')
        assert_equal(len(list(splitter)), 1)

    def test_legal_level_is_correctly_read(self):
        splitter = Splitter('*DEBUG* My message details')
        self._verify_message(splitter, 'My message details', 'DEBUG')
        assert_equal(len(list(splitter)), 1)

    def test_space_after_level_is_optional(self):
        splitter = Splitter('*WARN*No space!')
        self._verify_message(splitter, 'No space!', 'WARN')
        assert_equal(len(list(splitter)), 1)

    def test_it_is_possible_to_define_multiple_levels(self):
        splitter = Splitter('*WARN* WARNING!\n'
                            '*TRACE*msg')
        self._verify_message(splitter, 'WARNING!', 'WARN')
        self._verify_message(splitter, 'msg', 'TRACE', index=1)
        assert_equal(len(list(splitter)), 2)

    def test_html_flag_should_be_parsed_correctly_and_uses_info_level(self):
        splitter = Splitter('*HTML* <b>Hello</b>')
        self._verify_message(splitter, '<b>Hello</b>', level='INFO', html=True)
        assert_equal(len(list(splitter)), 1)

    def test_default_level_for_first_message_is_info(self):
        splitter = Splitter('<img src="foo bar">\n'
                                   '*DEBUG*bar foo')
        self._verify_message(splitter, '<img src="foo bar">')
        self._verify_message(splitter, 'bar foo', 'DEBUG', index=1)
        assert_equal(len(list(splitter)), 2)

    def test_timestamp_given_as_integer(self):
        now = int(time.time())
        splitter = Splitter('*INFO:xxx* No timestamp\n'
                            '*INFO:0* Epoch\n'
                            '*HTML:%d*X' % (now*1000))
        self._verify_message(splitter, '*INFO:xxx* No timestamp')
        self._verify_message(splitter, 'Epoch', timestamp=0, index=1)
        self._verify_message(splitter, html=True, timestamp=now, index=2)
        assert_equal(len(list(splitter)), 3)

    def test_timestamp_given_as_float(self):
        splitter = Splitter('*INFO:1x2* No timestamp\n'
                            '*HTML:1000.123456789* X\n'
                            '*INFO:12345678.9*X')
        self._verify_message(splitter, '*INFO:1x2* No timestamp')
        self._verify_message(splitter, html=True, timestamp=1, index=1)
        self._verify_message(splitter, timestamp=12345.679, index=2)
        assert_equal(len(list(splitter)), 3)

    def _verify_message(self, splitter, msg='X', level='INFO', html=False,
                        timestamp=None, index=0):
        message = list(splitter)[index]
        assert_equal(message.message, msg)
        assert_equal(message.level, level)
        assert_equal(message.html, html)
        if timestamp:
            assert_equal(message.timestamp,
                          format_time(timestamp, millissep='.'))


if __name__ == '__main__':
    unittest.main()
